@@ -5,6 +5,7 @@
 #include <gio/gio.h>
 
 #include "manifest.h"
+#include "utils.h"
 
 #define R_BUNDLE_ERROR r_bundle_error_quark()
 GQuark r_bundle_error_quark(void);
@@ -35,6 +36,24 @@ typedef struct {
 	STACK_OF(X509) *verified_chain;
 } RaucBundle;
 
+typedef enum {
+	CHECK_BUNDLE_DEFAULT       = 0,
+	/* skip BIT(0) for now to avoid interpreting a TRUE as NO_VERIFY by mistake */
+	CHECK_BUNDLE_NO_VERIFY     = BIT(1),      // If not set the bundle signature
+	                                          // will be verified, if set this
+	                                          // step will be skipped.
+	CHECK_BUNDLE_NO_CHECK_TIME = BIT(2),      // If set, X509_V_FLAG_NO_CHECK_TIME
+	                                          // is passed to openssl to suppress
+	                                          // checking the validity period of
+	                                          // certificates and CRLs against
+	                                          // the current time.
+	CHECK_BUNDLE_TRUST_ENV     = BIT(3),      // If set, the runtime environment
+	                                          // is fully trusted and no attempts
+	                                          // will be made to protect against
+	                                          // concurrent modification of the
+	                                          // bundle.
+} CheckBundleParams;
+
 /**
  * Create a bundle.
  *
@@ -56,13 +75,12 @@ G_GNUC_WARN_UNUSED_RESULT;
  * @param bundle return location for a RaucBundle struct.
  *               This will contain all bundle information obtained by
  *               check_bundle
- * @param verify If set to true the bundle signature will also be verified, if
- *               set to FALSE this step will be skipped
+ * @param params bit-field enum CheckBundleParams with additional flags for the check
  * @param error Return location for a GError
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean verify, GError **error)
+gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, CheckBundleParams params, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
 
 /**
@@ -103,6 +121,43 @@ G_GNUC_WARN_UNUSED_RESULT;
 gboolean resign_bundle(RaucBundle *bundle, const gchar *outpath, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
 
+/**
+ * Replace a bundle signature.
+ *
+ * This will create a copy of a bundle with a new given signature
+ * but unmodified content.
+ *
+ * @param bundle RaucBundle struct as returned by check_bundle()
+ * @param insig filename of the signature for replacement
+ * @param outpath filename of the output bundle
+ * @param params bit-field enum CheckBundleParams with additional flags for the check
+ * @param error Return location for a GError
+ *
+ * Note that check_bundle() must be called prior to this, to obtain a
+ * RaucBundle struct.
+ *
+ * @return TRUE on success, FALSE if an error occurred
+ */
+gboolean replace_signature(RaucBundle *bundle, const gchar *insig, const gchar *outpath, CheckBundleParams params, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Extract a bundle signature.
+ *
+ * This will extract the bundle signature into a given file.
+ *
+ * @param bundle RaucBundle struct as returned by check_bundle()
+ * @param outputsig file to extract signature
+ * @param error Return location for a GError
+ *
+ * Note that check_bundle() must be called prior to this, to obtain a
+ * RaucBundle struct.
+ *
+ * @return TRUE on success, FALSE if an error occurred
+ */
+gboolean extract_signature(RaucBundle *bundle, const gchar *outputsig, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
 /**
  * Extract a bundle.
  *
@@ -120,24 +175,6 @@ G_GNUC_WARN_UNUSED_RESULT;
 gboolean extract_bundle(RaucBundle *bundle, const gchar *outputdir, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
 
-/**
- * Extract a single file from a bundle.
- *
- * This will extract a single file into a given directory.
- *
- * Note that check_bundle() must be called prior to this, to obtain a
- * RaucBundle struct.
- *
- * @param bundle RaucBundle struct as returned by check_bundle()
- * @param outputdir directory to extract the file into
- * @param file filename of file to extract from bundle
- * @param error Return location for a GError
- *
- * @return TRUE on success, FALSE if an error occurred
- */
-gboolean extract_file_from_bundle(RaucBundle *bundle, const gchar *outputdir, const gchar *file, GError **error)
-G_GNUC_WARN_UNUSED_RESULT;
-
 /**
  * Extract and load the manifest from a bundle.
  *
