@@ -20,11 +20,18 @@ GQuark r_bootchooser_error_quark(void)
 #define BAREBOX_STATE_PRIORITY_PRIMARY	20
 #define UBOOT_FWSETENV_NAME "fw_setenv"
 #define UBOOT_FWPRINTENV_NAME "fw_printenv"
-#define UBOOT_DEFAULT_ATTEMPTS		"3"
-#define UBOOT_ATTEMPTS_PRIMARY		"3"
+#define UBOOT_DEFAULT_ATTEMPTS  3
+#define UBOOT_ATTEMPTS_PRIMARY  3
 #define EFIBOOTMGR_NAME "efibootmgr"
 #define GRUB_EDITENV "grub-editenv"
 
+static const gchar *supported_bootloaders[] = {"barebox", "grub", "uboot", "efi", "custom", "noop", NULL};
+
+gboolean r_boot_is_supported_bootloader(const gchar *bootloader)
+{
+	return g_strv_contains(supported_bootloaders, bootloader);
+}
+
 static GString *bootchooser_order_primay(RaucSlot *slot)
 {
 	GString *order = NULL;
@@ -80,6 +87,10 @@ static gboolean barebox_state_get(const gchar* bootname, BareboxSlotState *bb_st
 	g_ptr_array_add(args, g_strdup_printf(BOOTSTATE_PREFIX ".%s.priority", bootname));
 	g_ptr_array_add(args, g_strdup("-g"));
 	g_ptr_array_add(args, g_strdup_printf(BOOTSTATE_PREFIX ".%s.remaining_attempts", bootname));
+	if (r_context()->config->system_bb_dtbpath) {
+		g_ptr_array_add(args, g_strdup("-i"));
+		g_ptr_array_add(args, g_strdup(r_context()->config->system_bb_dtbpath));
+	}
 	g_ptr_array_add(args, NULL);
 
 	sub = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_STDOUT_PIPE, &ierror);
@@ -168,6 +179,10 @@ static gboolean barebox_state_set(GPtrArray *pairs, GError **error)
 		g_ptr_array_add(args, g_strdup("-s"));
 		g_ptr_array_add(args, g_strdup(pairs->pdata[i]));
 	}
+	if (r_context()->config->system_bb_dtbpath) {
+		g_ptr_array_add(args, g_strdup("-i"));
+		g_ptr_array_add(args, g_strdup(r_context()->config->system_bb_dtbpath));
+	}
 	g_ptr_array_add(args, NULL);
 
 	sub = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_NONE, &ierror);
@@ -757,7 +772,7 @@ static gboolean uboot_get_state(RaucSlot* slot, gboolean *good, GError **error)
 		g_propagate_error(error, ierror);
 		return FALSE;
 	}
-	*good = (atoi(attempts->str) > 0) ? TRUE : FALSE;
+	*good = (g_ascii_strtoull(attempts->str, NULL, 16) > 0) ? TRUE : FALSE;
 
 	return TRUE;
 }
@@ -767,6 +782,8 @@ static gboolean uboot_set_state(RaucSlot *slot, gboolean good, GError **error)
 {
 	GError *ierror = NULL;
 	g_autofree gchar *key = NULL;
+	g_autofree gchar *val = NULL;
+	gint attempts = 0;
 
 	g_return_val_if_fail(slot, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
@@ -810,7 +827,15 @@ set_left:
 
 	key = g_strdup_printf("BOOT_%s_LEFT", slot->bootname);
 
-	if (!uboot_env_set(key, good ? UBOOT_DEFAULT_ATTEMPTS : "0", &ierror)) {
+	if (good) {
+		attempts = r_context()->config->boot_default_attempts;
+		if (attempts <= 0)
+			attempts = UBOOT_DEFAULT_ATTEMPTS;
+	}
+
+	val = g_strdup_printf("%x", attempts);
+
+	if (!uboot_env_set(key, val, &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
 	}
@@ -854,7 +879,7 @@ static RaucSlot* uboot_get_primary(GError **error)
 				return NULL;
 			}
 
-			if (atoi(attempts->str) <= 0)
+			if (g_ascii_strtoull(attempts->str, NULL, 16) <= 0)
 				continue;
 
 			primary = slot;
@@ -884,6 +909,8 @@ static gboolean uboot_set_primary(RaucSlot *slot, GError **error)
 	g_auto(GStrv) bootnames = NULL;
 	GError *ierror = NULL;
 	g_autofree gchar *key = NULL;
+	g_autofree gchar *val = NULL;
+	gint attempts;
 
 	g_return_val_if_fail(slot, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
@@ -915,7 +942,13 @@ static gboolean uboot_set_primary(RaucSlot *slot, GError **error)
 
 	key = g_strdup_printf("BOOT_%s_LEFT", slot->bootname);
 
-	if (!uboot_env_set(key, UBOOT_ATTEMPTS_PRIMARY, &ierror)) {
+	attempts = r_context()->config->boot_attempts_primary;
+	if (attempts <= 0)
+		attempts = UBOOT_ATTEMPTS_PRIMARY;
+
+	val = g_strdup_printf("%d", attempts);
+
+	if (!uboot_env_set(key, val, &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
 	}
@@ -1187,7 +1220,7 @@ static gboolean efi_set_temp_primary(RaucSlot *slot, GError **error)
 }
 
 /* Deletes given slot from efi bootorder list.
- * Prepends it to bootorder list if prepend arguemnt is set to TRUE */
+ * Prepends it to bootorder list if prepend argument is set to TRUE */
 static gboolean efi_modify_persistent_bootorder(RaucSlot *slot, gboolean prepend, GError **error)
 {
 	GList *entries = NULL;
@@ -1454,7 +1487,6 @@ static gboolean custom_backend_set(const gchar *cmd, const gchar *bootname, cons
 {
 	g_autoptr(GSubprocess) sub = NULL;
 	GError *ierror = NULL;
-	g_autoptr(GBytes) stdout_buf = NULL;
 	gchar *backend_name = r_context()->config->custom_bootloader_backend;
 
 	g_return_val_if_fail(cmd, FALSE);
