@@ -79,6 +79,12 @@
 #ifndef XFS_SUPER_MAGIC
 #define XFS_SUPER_MAGIC 0x58465342
 #endif
+#ifndef ZFS_SUPER_MAGIC
+/* Taken from https://github.com/openzfs/zfs/blob/master/include/sys/fs/zfs.h#L1198 */
+#define ZFS_SUPER_MAGIC 0x2fc12fc1
+#endif
+
+#define MAX_BUNDLE_SIGNATURE_SIZE 0x10000
 
 GQuark
 r_bundle_error_quark(void)
@@ -499,7 +505,7 @@ static gboolean sign_bundle(const gchar *bundlename, RaucManifest *manifest, GEr
 		int bundlefd = g_file_descriptor_based_get_fd(G_FILE_DESCRIPTOR_BASED(bundleoutstream));
 		guint8 salt[32] = {0};
 		guint8 hash[32] = {0};
-		off_t combined_size = 0;
+		uint64_t combined_size = 0;
 		guint64 verity_size = 0;
 
 		g_print("Creating bundle in 'verity' format\n");
@@ -524,6 +530,13 @@ static gboolean sign_bundle(const gchar *bundlename, RaucManifest *manifest, GEr
 					"squashfs size (%"G_GUINT64_FORMAT ") is not a multiple of 4096 bytes", offset);
 			return FALSE;
 		}
+		if (offset <= 4096) {
+			g_set_error(error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_CREATE_SIG,
+					"squashfs size (%"G_GUINT64_FORMAT ") must be larger than 4096 bytes", offset);
+			return FALSE;
+		}
 		if (verity_create_or_verify_hash(0, bundlefd, offset/4096, &combined_size, hash, salt) != 0) {
 			g_set_error(error,
 					R_SIGNATURE_ERROR,
@@ -532,7 +545,7 @@ static gboolean sign_bundle(const gchar *bundlename, RaucManifest *manifest, GEr
 			return FALSE;
 		}
 		/* for a squashfs <= 4096 bytes, we don't have a hash table */
-		g_assert(combined_size*4096 >= (off_t)offset);
+		g_assert(combined_size*4096 > (uint64_t)offset);
 		verity_size = combined_size*4096 - offset;
 		g_assert(verity_size % 4096 == 0);
 
@@ -612,7 +625,7 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		goto out;
 	}
 
-	res = update_manifest_checksums(manifest, contentdir, &ierror);
+	res = sync_manifest_with_contentdir(manifest, contentdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -773,6 +786,9 @@ out:
 
 static gboolean image_is_archive(RaucImage* image)
 {
+	g_return_val_if_fail(image, FALSE);
+	g_return_val_if_fail(image->filename, FALSE);
+
 	if (g_pattern_match_simple("*.tar*", image->filename) ||
 	    g_pattern_match_simple("*.catar", image->filename)) {
 		return TRUE;
@@ -810,7 +826,7 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 	}
 
 	if (g_file_test(storepath, G_FILE_TEST_EXISTS)) {
-		g_warning("Store path '%s' already exists, appending new chunks", outbundle);
+		g_warning("Store path '%s' already exists, appending new chunks", storepath);
 	}
 
 	/* Set up tmp dir for conversion */
@@ -852,6 +868,9 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 
 		imgpath = g_build_filename(contentdir, image->filename, NULL);
 
+		if (!image->filename)
+			continue;
+
 		if (image_is_archive(image)) {
 			idxfile = g_strconcat(image->filename, ".caidx", NULL);
 			idxpath = g_build_filename(contentdir, idxfile, NULL);
@@ -1101,7 +1120,6 @@ static gboolean check_bundle_access(int bundle_fd, GError **error)
 			case JFFS2_SUPER_MAGIC:
 			case MSDOS_SUPER_MAGIC:
 			case NTFS_SB_MAGIC:
-			case RAMFS_MAGIC:
 			case ROMFS_MAGIC:
 			case SQUASHFS_MAGIC:
 			case UDF_SUPER_MAGIC:
@@ -1110,12 +1128,14 @@ static gboolean check_bundle_access(int bundle_fd, GError **error)
 			/* these are prepared by root */
 			case HOSTFS_SUPER_MAGIC:
 			case OVERLAYFS_SUPER_MAGIC:
+			case RAMFS_MAGIC:
 			case TMPFS_MAGIC:
 			case UBIFS_SUPER_MAGIC:
+			case ZFS_SUPER_MAGIC:
 				mount_checked = TRUE;
 				break;
 			default:
-				g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_UNSAFE, "bundle is stored on an unkown filesystem (type=%0jx)", (uintmax_t)bundle_statfs.f_type);
+				g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_UNSAFE, "bundle is stored on an unknown filesystem (type=%0jx)", (uintmax_t)bundle_statfs.f_type);
 				res = FALSE;
 				goto out;
 				break;
@@ -1224,7 +1244,7 @@ static gboolean enforce_bundle_exclusive(int bundle_fd, GError **error)
 			/* taking ownership was successful, the relevant error is ierror_check */
 			g_propagate_error(error, ierror_check);
 		} else {
-			/* taking ownership was unsucessful, the relevant error is ierror_take */
+			/* taking ownership was unsuccessful, the relevant error is ierror_take */
 			g_clear_error(&ierror_check);
 			g_propagate_prefixed_error(error, ierror_take, "failed to take ownership of bundle: ");
 			ierror_take = NULL;
@@ -1240,14 +1260,141 @@ out:
 	return res;
 }
 
-gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean verify, GError **error)
+static gboolean open_local_bundle(RaucBundle *bundle, GError **error)
 {
+	gboolean res = FALSE;
 	GError *ierror = NULL;
 	g_autoptr(GFile) bundlefile = NULL;
 	g_autoptr(GFileInfo) bundleinfo = NULL;
 	guint64 sigsize;
 	goffset offset;
+
+	g_return_val_if_fail(bundle != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	g_assert_null(bundle->stream);
+
+	bundlefile = g_file_new_for_path(bundle->path);
+	bundle->stream = G_INPUT_STREAM(g_file_read(bundlefile, NULL, &ierror));
+	if (bundle->stream == NULL) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to open bundle for reading: ");
+		res = FALSE;
+		goto out;
+	}
+
+	bundleinfo = g_file_input_stream_query_info(
+			G_FILE_INPUT_STREAM(bundle->stream),
+			G_FILE_ATTRIBUTE_STANDARD_TYPE,
+			NULL, &ierror);
+	if (bundleinfo == NULL) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to query bundle file info: ");
+		res = FALSE;
+		goto out;
+	}
+
+	if (g_file_info_get_file_type(bundleinfo) != G_FILE_TYPE_REGULAR) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_UNSAFE,
+				"Bundle is not a regular file");
+		res = FALSE;
+		goto out;
+	}
+
+	res = input_stream_check_bundle_identifier(bundle->stream, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to check bundle identifier: ");
+		goto out;
+	}
+
+	offset = sizeof(sigsize);
+	res = g_seekable_seek(G_SEEKABLE(bundle->stream),
+			-offset, G_SEEK_END, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to seek to end of bundle: ");
+		goto out;
+	}
+	offset = g_seekable_tell(G_SEEKABLE(bundle->stream));
+
+	res = input_stream_read_uint64_all(bundle->stream,
+			&sigsize, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to read signature size from bundle: ");
+		goto out;
+	}
+
+	if (sigsize == 0) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
+				"Signature size is 0");
+		res = FALSE;
+		goto out;
+	}
+	/* sanity check: signature should be smaller than bundle size */
+	if (sigsize > (guint64)offset) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
+				"Signature size (%"G_GUINT64_FORMAT ") exceeds bundle size", sigsize);
+		res = FALSE;
+		goto out;
+	}
+	/* sanity check: signature should be smaller than 64KiB */
+	if (sigsize > MAX_BUNDLE_SIGNATURE_SIZE) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
+				"Signature size (%"G_GUINT64_FORMAT ") exceeds 64KiB", sigsize);
+		res = FALSE;
+		goto out;
+	}
+
+	offset -= sigsize;
+	if (offset % 4096) {
+		g_message(
+				"Payload size (%"G_GUINT64_FORMAT ") is not a multiple of 4KiB. "
+				"See https://rauc.readthedocs.io/en/latest/faq.html#what-causes-a-payload-size-that-is-not-a-multiple-of-4kib",
+				offset);
+	}
+	bundle->size = offset;
+
+	res = g_seekable_seek(G_SEEKABLE(bundle->stream),
+			offset, G_SEEK_SET, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to seek to start of bundle signature: ");
+		goto out;
+	}
+
+	res = input_stream_read_bytes_all(bundle->stream,
+			&bundle->sigdata, sigsize, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to read signature from bundle: ");
+		goto out;
+	}
+
+out:
+	return res;
+}
+
+gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, CheckBundleParams params, GError **error)
+{
+	GError *ierror = NULL;
 	gboolean res = FALSE;
+	gboolean verify = !(params & CHECK_BUNDLE_NO_VERIFY);
 	g_autoptr(RaucBundle) ibundle = g_new0(RaucBundle, 1);
 	g_autoptr(GBytes) manifest_bytes = NULL;
 	gchar *bundlescheme = NULL;
@@ -1255,18 +1402,32 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 
 	g_return_val_if_fail(bundlename, FALSE);
 	g_return_val_if_fail(bundle != NULL && *bundle == NULL, FALSE);
+	g_return_val_if_fail(!(params & TRUE), FALSE); /* protect against passing TRUE as the params enum */
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
 	r_context_begin_step("check_bundle", "Checking bundle", verify);
 
+	if (verify && !r_context()->config->keyring_path && !r_context()->config->keyring_directory) {
+		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_KEYRING, "No keyring file or directory provided");
+		res = FALSE;
+		goto out;
+	}
+
 	ibundle->verification_disabled = !verify;
 
 	/* Download Bundle to temporary location if remote URI is given */
 	bundlescheme = g_uri_parse_scheme(bundlename);
 	if (is_remote_scheme(bundlescheme)) {
 #if ENABLE_NETWORK
+		g_autofree gchar *tmpdir = g_dir_make_tmp("rauc-XXXXXX", &ierror);
+		if (tmpdir == NULL) {
+			g_propagate_prefixed_error(error, ierror, "Failed to create tmp dir: ");
+			res = FALSE;
+			goto out;
+		}
+
 		ibundle->origpath = g_strdup(bundlename);
-		ibundle->path = g_build_filename(g_get_tmp_dir(), "_download.raucb", NULL);
+		ibundle->path = g_build_filename(tmpdir, "download.raucb", NULL);
 
 		g_message("Remote URI detected, downloading bundle to %s...", ibundle->path);
 		res = download_file(ibundle->path, ibundle->origpath, r_context()->config->max_bundle_download_size, &ierror);
@@ -1297,123 +1458,11 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 		}
 	}
 
-	if (verify && !r_context()->config->keyring_path && !r_context()->config->keyring_directory) {
-		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_KEYRING, "No keyring file or directory provided");
-		res = FALSE;
-		goto out;
-	}
-
 	g_message("Reading bundle: %s", ibundle->path);
 
-	bundlefile = g_file_new_for_path(ibundle->path);
-	ibundle->stream = G_INPUT_STREAM(g_file_read(bundlefile, NULL, &ierror));
-	if (ibundle->stream == NULL) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to open bundle for reading: ");
-		res = FALSE;
-		goto out;
-	}
-
-	bundleinfo = g_file_input_stream_query_info(
-			G_FILE_INPUT_STREAM(ibundle->stream),
-			G_FILE_ATTRIBUTE_STANDARD_TYPE,
-			NULL, &ierror);
-	if (bundleinfo == NULL) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to query bundle file info: ");
-		res = FALSE;
-		goto out;
-	}
-
-	if (g_file_info_get_file_type(bundleinfo) != G_FILE_TYPE_REGULAR) {
-		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_UNSAFE,
-				"Bundle is not a regular file");
-		res = FALSE;
-		goto out;
-	}
-
-	res = input_stream_check_bundle_identifier(ibundle->stream, &ierror);
+	res = open_local_bundle(ibundle, &ierror);
 	if (!res) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to check bundle identifier: ");
-		goto out;
-	}
-
-	offset = sizeof(sigsize);
-	res = g_seekable_seek(G_SEEKABLE(ibundle->stream),
-			-offset, G_SEEK_END, NULL, &ierror);
-	if (!res) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to seek to end of bundle: ");
-		goto out;
-	}
-	offset = g_seekable_tell(G_SEEKABLE(ibundle->stream));
-
-	res = input_stream_read_uint64_all(ibundle->stream,
-			&sigsize, NULL, &ierror);
-	if (!res) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to read signature size from bundle: ");
-		goto out;
-	}
-
-	if (sigsize == 0) {
-		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
-				"Signature size is 0");
-		res = FALSE;
-		goto out;
-	}
-	/* sanity check: signature should be smaller than bundle size */
-	if (sigsize > (guint64)offset) {
-		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
-				"Signature size (%"G_GUINT64_FORMAT ") exceeds bundle size", sigsize);
-		res = FALSE;
-		goto out;
-	}
-	/* sanity check: signature should be smaller than 64kiB */
-	if (sigsize > 0x4000000) {
-		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
-				"Signature size (%"G_GUINT64_FORMAT ") exceeds 64KiB", sigsize);
-		res = FALSE;
-		goto out;
-	}
-
-	offset -= sigsize;
-	if (offset % 4096) {
-		g_message(
-				"Payload size (%"G_GUINT64_FORMAT ") is not a multiple of 4KiB. "
-				"See https://rauc.readthedocs.io/en/latest/faq.html#what-causes-a-payload-size-that-is-not-a-multiple-of-4kib",
-				offset);
-	}
-	ibundle->size = offset;
-
-	res = g_seekable_seek(G_SEEKABLE(ibundle->stream),
-			offset, G_SEEK_SET, NULL, &ierror);
-	if (!res) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to seek to start of bundle signature: ");
-		goto out;
-	}
-
-	res = input_stream_read_bytes_all(ibundle->stream,
-			&ibundle->sigdata, sigsize, NULL, &ierror);
-	if (!res) {
-		g_propagate_prefixed_error(
-				error,
-				ierror,
-				"Failed to read signature from bundle: ");
+		g_propagate_error(error, ierror);
 		goto out;
 	}
 
@@ -1426,13 +1475,19 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 	if (verify) {
 		CMS_ContentInfo *cms = NULL;
 		X509_STORE *store = setup_x509_store(NULL, NULL, &ierror);
+		X509_VERIFY_PARAM *param = NULL;
 		if (!store) {
 			g_propagate_error(error, ierror);
 			res = FALSE;
 			goto out;
 		}
+		param = X509_STORE_get0_param(store);
 
 		g_message("Verifying bundle signature... ");
+
+		if (params & CHECK_BUNDLE_NO_CHECK_TIME)
+			X509_VERIFY_PARAM_set_flags(param, X509_V_FLAG_NO_CHECK_TIME);
+
 		if (detached) {
 			int fd = g_file_descriptor_based_get_fd(G_FILE_DESCRIPTOR_BASED(ibundle->stream));
 
@@ -1450,8 +1505,8 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 			}
 			ibundle->exclusive_verified = TRUE;
 
-			/* the squashfs image size is in offset */
-			res = cms_verify_fd(fd, ibundle->sigdata, offset, store, &cms, &ierror);
+			/* the squashfs image size is in ibundle->size */
+			res = cms_verify_fd(fd, ibundle->sigdata, ibundle->size, store, &cms, &ierror);
 			if (!res) {
 				g_propagate_error(error, ierror);
 				goto out;
@@ -1460,6 +1515,7 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 			ibundle->payload_verified = TRUE;
 		} else {
 			int fd = g_file_descriptor_based_get_fd(G_FILE_DESCRIPTOR_BASED(ibundle->stream));
+			gboolean trust_env = (params & CHECK_BUNDLE_TRUST_ENV);
 
 			if (!(r_context()->config->bundle_formats_mask & 1 << R_MANIFEST_FORMAT_VERITY)) {
 				g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_FORMAT,
@@ -1468,7 +1524,7 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 				goto out;
 			}
 
-			if (!check_bundle_access(fd, &ierror)) {
+			if (!trust_env && !check_bundle_access(fd, &ierror)) {
 				ibundle->exclusive_check_error = g_strdup(ierror->message);
 				g_clear_error(&ierror);
 			} else {
@@ -1591,6 +1647,198 @@ out:
 	return res;
 }
 
+gboolean replace_signature(RaucBundle *bundle, const gchar *insig, const gchar *outpath, CheckBundleParams params, GError **error)
+{
+	g_autoptr(RaucManifest) manifest = NULL;
+	g_autoptr(RaucBundle) outbundle = NULL;
+	g_autoptr(GFile) bundleoutfile = NULL;
+	GFileIOStream* bundlestream = NULL;
+	GOutputStream* bundleoutstream = NULL;
+	g_autoptr(GBytes) sig = NULL;
+	gchar* keyringpath = NULL;
+	gchar* keyringdirectory = NULL;
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+	gsize sigsize;
+
+	g_return_val_if_fail(bundle != NULL, FALSE);
+	g_return_val_if_fail(outpath != NULL, FALSE);
+	g_return_val_if_fail(insig != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	r_context_begin_step("replace_signature", "Replacing bundle signature", 5);
+
+	res = check_bundle_payload(bundle, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	res = load_manifest_from_bundle(bundle, &manifest, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	if (manifest->bundle_format == R_MANIFEST_FORMAT_PLAIN) {
+		g_print("Reading bundle in 'plain' format\n");
+	} else if (manifest->bundle_format == R_MANIFEST_FORMAT_VERITY) {
+		g_print("Reading bundle in 'verity' format\n");
+	} else {
+		g_error("unsupported bundle format");
+		res = FALSE;
+		goto out;
+	}
+
+	sig = read_file(insig, &ierror);
+	if (!sig) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to read signature file: ");
+		res = FALSE;
+		goto out;
+	}
+
+	res = truncate_bundle(bundle->path, outpath, bundle->size, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	bundleoutfile = g_file_new_for_path(outpath);
+	bundlestream = g_file_open_readwrite(bundleoutfile, NULL, &ierror);
+	if (!bundlestream) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to open new bundle for adding signature: ");
+		res = FALSE;
+		goto out;
+	}
+
+	bundleoutstream = g_io_stream_get_output_stream(G_IO_STREAM(bundlestream));
+
+	res = g_seekable_seek(G_SEEKABLE(bundleoutstream),
+			0, G_SEEK_END, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to seek to end of new bundle: ");
+		goto out;
+	}
+
+	res = output_stream_write_bytes_all(bundleoutstream, sig, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to append signature to temporary bundle: ");
+		goto out;
+	}
+
+	sigsize = g_bytes_get_size(sig);
+	res = output_stream_write_uint64_all(bundleoutstream, (guint64)sigsize, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to append signature size to new bundle: ");
+		goto out;
+	}
+
+	/* Necessary to release associated fd before perform check_bundle */
+	g_clear_object(&bundlestream);
+
+	/*
+	 * If signing_keyringpath is given, replace config->keyring_path, so we can
+	 * use check_bundle() as is.
+	 */
+	if (r_context()->signing_keyringpath) {
+		keyringpath = r_context()->config->keyring_path;
+		keyringdirectory = r_context()->config->keyring_directory;
+		r_context()->config->keyring_path = r_context()->signing_keyringpath;
+		r_context()->config->keyring_directory = NULL;
+	}
+
+	/* Let the user control verification by optionally providing a keyring. */
+	if (r_context()->config->keyring_path || r_context()->config->keyring_directory) {
+		g_message("Keyring given, enabling signature verification");
+		params &= ~CHECK_BUNDLE_NO_VERIFY;
+	} else {
+		g_message("No keyring given, disabling signature verification");
+		params |= CHECK_BUNDLE_NO_VERIFY;
+	}
+
+	res = check_bundle(outpath, &outbundle, params, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to verify the new bundle: ");
+		goto out;
+	}
+
+	res = TRUE;
+out:
+	/* Remove output file on error */
+	if (!res &&
+	    g_file_test(outpath, G_FILE_TEST_IS_REGULAR) &&
+	    !g_error_matches(ierror, G_FILE_ERROR, G_FILE_ERROR_EXIST))
+		if (g_remove(outpath) != 0)
+			g_warning("failed to remove %s", outpath);
+
+	if (bundlestream)
+		g_clear_object(&bundlestream);
+
+	/* Restore saved paths if necessary */
+	if (keyringpath || keyringdirectory) {
+		r_context()->config->keyring_path = keyringpath;
+		r_context()->config->keyring_directory = keyringdirectory;
+	}
+
+	r_context_end_step("replace_signature", res);
+	return res;
+}
+
+gboolean extract_signature(RaucBundle *bundle, const gchar *outputsig, GError **error)
+{
+	GError *ierror = NULL;
+	g_autoptr(GFile) sigfile = NULL;
+	g_autoptr(GOutputStream) sigoutstream = NULL;
+	gboolean res = FALSE;
+
+	g_return_val_if_fail(bundle != NULL, FALSE);
+	g_return_val_if_fail(outputsig != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	r_context_begin_step("extract_signature", "Extracting bundle signature", 0);
+
+	sigfile = g_file_new_for_path(outputsig);
+	sigoutstream = (GOutputStream*)g_file_create(sigfile, G_FILE_CREATE_PRIVATE, NULL, &ierror);
+	if (sigoutstream == NULL) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to create file to store signature: ");
+		goto out;
+	}
+
+	if (!output_stream_write_bytes_all(sigoutstream, bundle->sigdata, NULL, &ierror)) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to write signature to file: ");
+		goto out;
+	}
+
+	res = TRUE;
+out:
+	r_context_end_step("extract_signature", res);
+	return res;
+}
+
 gboolean extract_bundle(RaucBundle *bundle, const gchar *outputdir, GError **error)
 {
 	GError *ierror = NULL;
@@ -1618,30 +1866,6 @@ out:
 	return res;
 }
 
-gboolean extract_file_from_bundle(RaucBundle *bundle, const gchar *outputdir, const gchar *file, GError **error)
-{
-	GError *ierror = NULL;
-	gboolean res = FALSE;
-
-	g_return_val_if_fail(bundle != NULL, FALSE);
-
-	res = check_bundle_payload(bundle, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
-	}
-
-	res = unsquashfs(g_file_descriptor_based_get_fd(G_FILE_DESCRIPTOR_BASED(bundle->stream)), outputdir, file, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
-	}
-
-	res = TRUE;
-out:
-	return res;
-}
-
 gboolean load_manifest_from_bundle(RaucBundle *bundle, RaucManifest **manifest, GError **error)
 {
 	g_autofree gchar* tmpdir = NULL;
@@ -1722,7 +1946,7 @@ gboolean mount_bundle(RaucBundle *bundle, GError **error)
 	}
 
 	if (!(bundle->signature_verified || bundle->verification_disabled))
-		g_error("bundle signature must be verfied before mounting");
+		g_error("bundle signature must be verified before mounting");
 
 	g_message("Mounting bundle '%s' to '%s'", bundle->path, mount_point);
 
@@ -1738,7 +1962,7 @@ gboolean mount_bundle(RaucBundle *bundle, GError **error)
 		g_autofree gchar* manifestpath = NULL;
 
 		if (!(bundle->payload_verified || bundle->verification_disabled))
-			g_error("bundle payload must be verfied before mounting for plain bundles");
+			g_error("bundle payload must be verified before mounting for plain bundles");
 
 		res = r_mount_bundle(loopname, mount_point, &ierror);
 		if (!res) {
@@ -1851,13 +2075,19 @@ out:
 
 void free_bundle(RaucBundle *bundle)
 {
-	g_return_if_fail(bundle);
+	if (!bundle)
+		return;
 
 	/* In case of a temporary download artifact, remove it. */
-	if (bundle->origpath)
+	if (bundle->origpath) {
+		g_autofree gchar *tmpdir = g_path_get_dirname(bundle->path);
 		if (g_remove(bundle->path) != 0) {
 			g_warning("failed to remove download artifact %s: %s\n", bundle->path, g_strerror(errno));
 		}
+		if (g_rmdir(tmpdir) != 0) {
+			g_warning("failed to remove download directory %s: %s\n", tmpdir, g_strerror(errno));
+		}
+	}
 
 	g_free(bundle->path);
 	if (bundle->stream)
