@@ -42,7 +42,7 @@ static void install_args_update(RaucInstallArgs *args, const gchar *msg)
 	g_main_context_invoke(NULL, args->notify, args);
 }
 
-static gchar *resolve_loop_device(const gchar *devicepath)
+static gchar *resolve_loop_device(const gchar *devicepath, GError **error)
 {
 	g_autofree gchar *devicename = NULL;
 	g_autofree gchar *syspath = NULL;
@@ -57,8 +57,10 @@ static gchar *resolve_loop_device(const gchar *devicepath)
 
 	content = read_file_str(syspath, &ierror);
 	if (!content) {
-		g_message("%s", ierror->message);
-		g_clear_error(&ierror);
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Error getting loop backing_file for %s: ", devicepath);
 		return NULL;
 	}
 
@@ -74,6 +76,7 @@ gboolean determine_slot_states(GError **error)
 	GHashTableIter iter;
 	RaucSlot *slot;
 	gboolean res = FALSE;
+	GError *ierror = NULL;
 
 	g_assert_nonnull(r_context()->config);
 
@@ -98,8 +101,14 @@ gboolean determine_slot_states(GError **error)
 	mountlist = g_unix_mounts_get(NULL);
 	for (GList *l = mountlist; l != NULL; l = l->next) {
 		GUnixMountEntry *m = (GUnixMountEntry*)l->data;
-		g_autofree gchar *devicepath = resolve_loop_device(g_unix_mount_get_device_path(m));
-		RaucSlot *s = find_config_slot_by_device(r_context()->config,
+		g_autofree gchar *devicepath = NULL;
+		RaucSlot *s;
+		devicepath = resolve_loop_device(g_unix_mount_get_device_path(m), &ierror);
+		if (!devicepath) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+		s = find_config_slot_by_device(r_context()->config,
 				devicepath);
 		if (s) {
 			/* We might have multiple mount entries matching the same device and thus the same slot.
@@ -152,6 +161,41 @@ gboolean determine_slot_states(GError **error)
 		}
 	}
 
+	if (!booted) {
+		gboolean extboot = FALSE;
+
+		if (g_strcmp0(r_context()->bootslot, "/dev/nfs") == 0) {
+			g_message("Detected nfs boot, ignoring missing active slot");
+			extboot = TRUE;
+		}
+
+		if (g_strcmp0(r_context()->bootslot, "_external_") == 0) {
+			g_message("Detected explicit external boot, ignoring missing active slot");
+			extboot = TRUE;
+		}
+
+		if (extboot) {
+			/* mark all as inactive */
+			g_debug("Marking all slots as 'inactive'");
+			for (GList *l = slotlist; l != NULL; l = l->next) {
+				RaucSlot *s = g_hash_table_lookup(r_context()->config->slots, l->data);
+				g_assert_nonnull(s);
+
+				s->state = ST_INACTIVE;
+			}
+
+			res = TRUE;
+			goto out;
+		}
+
+		g_set_error(
+				error,
+				R_SLOT_ERROR,
+				R_SLOT_ERROR_NO_SLOT_WITH_STATE_BOOTED,
+				"Did not find booted slot (matching '%s')", r_context()->bootslot);
+		goto out;
+	}
+
 	/* Determine active group members */
 	for (GList *l = slotlist; l != NULL; l = l->next) {
 		RaucSlot *s = g_hash_table_lookup(r_context()->config->slots, l->data);
@@ -167,27 +211,6 @@ gboolean determine_slot_states(GError **error)
 		}
 	}
 
-	if (!booted) {
-		if (g_strcmp0(r_context()->bootslot, "/dev/nfs") == 0) {
-			g_message("Detected nfs boot, ignoring missing active slot");
-			res = TRUE;
-			goto out;
-		}
-
-		if (g_strcmp0(r_context()->bootslot, "_external_") == 0) {
-			g_message("Detected explicit external boot, ignoring missing active slot");
-			res = TRUE;
-			goto out;
-		}
-
-		g_set_error_literal(
-				error,
-				R_SLOT_ERROR,
-				R_SLOT_ERROR_NO_SLOT_WITH_STATE_BOOTED,
-				"Did not find booted slot");
-		goto out;
-	}
-
 	res = TRUE;
 
 out:
@@ -200,19 +223,30 @@ gboolean determine_boot_states(GError **error)
 {
 	GHashTableIter iter;
 	RaucSlot *slot;
-	gchar *name;
-	GError *ierror = NULL;
+	gboolean had_errors = FALSE;
 
 	/* get boot state */
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
-	while (g_hash_table_iter_next(&iter, (gpointer*) &name, (gpointer*) &slot)) {
-		if (slot->bootname && !r_boot_get_state(slot, &slot->boot_good, &ierror)) {
-			g_propagate_error(error, ierror);
-			return FALSE;
+	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
+		GError *ierror = NULL;
+
+		if (!slot->bootname)
+			continue;
+
+		if (!r_boot_get_state(slot, &slot->boot_good, &ierror)) {
+			g_message("Failed to get boot state of %s: %s", slot->name, ierror->message);
+			had_errors = TRUE;
 		}
 	}
 
-	return TRUE;
+	if (had_errors)
+		g_set_error_literal(
+				error,
+				R_SLOT_ERROR,
+				R_SLOT_ERROR_NO_SLOT_WITH_STATE_BOOTED,
+				"Could not determine all boot states");
+
+	return !had_errors;
 }
 
 /* Returns NULL-teminated intern string array of all classes listed in
@@ -466,8 +500,8 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 	GHashTableIter iter;
 	RaucSlot *slot;
 	gint slotcnt = 0;
-	g_autofree gchar *targetlist = NULL;
-	g_autofree gchar *slotlist = NULL;
+	g_autoptr(GString) slots = g_string_sized_new(128);
+	g_autoptr(GString) target_slots = g_string_sized_new(128);
 
 	g_subprocess_launcher_setenv(launcher, "RAUC_SYSTEM_CONFIG", r_context()->configpath, TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_CURRENT_BOOTNAME", r_context()->bootslot, TRUE);
@@ -479,15 +513,14 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
 		gchar *varname;
-		gchar *tmp;
 		GHashTableIter iiter;
 		gpointer member;
 
 		slotcnt++;
 
-		tmp = g_strdup_printf("%s%i ", slotlist ? slotlist : "", slotcnt);
-		g_clear_pointer(&slotlist, g_free);
-		slotlist = tmp;
+		if (slots->len)
+			g_string_append_c(slots, ' ');
+		g_string_append_printf(slots, "%i", slotcnt);
 
 		g_hash_table_iter_init(&iiter, target_group);
 		while (g_hash_table_iter_next(&iiter, NULL, &member)) {
@@ -500,11 +533,11 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 				RaucImage *img = l->data;
 				if (g_str_equal(slot->sclass, img->slotclass)) {
 					varname = g_strdup_printf("RAUC_IMAGE_NAME_%i", slotcnt);
-					g_subprocess_launcher_setenv(launcher, varname, img->filename, TRUE);
+					g_subprocess_launcher_setenv(launcher, varname, img->filename ?: "", TRUE);
 					g_clear_pointer(&varname, g_free);
 
 					varname = g_strdup_printf("RAUC_IMAGE_DIGEST_%i", slotcnt);
-					g_subprocess_launcher_setenv(launcher, varname, img->checksum.digest, TRUE);
+					g_subprocess_launcher_setenv(launcher, varname, img->checksum.digest ?: "", TRUE);
 					g_clear_pointer(&varname, g_free);
 
 					varname = g_strdup_printf("RAUC_IMAGE_CLASS_%i", slotcnt);
@@ -515,9 +548,9 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 				}
 			}
 
-			tmp = g_strdup_printf("%s%i ", targetlist ? targetlist : "", slotcnt);
-			g_clear_pointer(&targetlist, g_free);
-			targetlist = tmp;
+			if (target_slots->len)
+				g_string_append_c(target_slots, ' ');
+			g_string_append_printf(target_slots, "%i", slotcnt);
 		}
 
 		varname = g_strdup_printf("RAUC_SLOT_NAME_%i", slotcnt);
@@ -545,8 +578,8 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 		g_clear_pointer(&varname, g_free);
 	}
 
-	g_subprocess_launcher_setenv(launcher, "RAUC_SLOTS", slotlist, TRUE);
-	g_subprocess_launcher_setenv(launcher, "RAUC_TARGET_SLOTS", targetlist, TRUE);
+	g_subprocess_launcher_setenv(launcher, "RAUC_SLOTS", slots->str, TRUE);
+	g_subprocess_launcher_setenv(launcher, "RAUC_TARGET_SLOTS", target_slots->str, TRUE);
 }
 
 static gboolean launch_and_wait_handler(RaucInstallArgs *args, gchar *update_source, gchar *handler_name, RaucManifest *manifest, GHashTable *target_group, GError **error)
@@ -745,6 +778,17 @@ static gboolean pre_install_checks(gchar* bundledir, GList *install_images, GHas
 		RaucImage *mfimage = l->data;
 		RaucSlot *dest_slot = g_hash_table_lookup(target_group, mfimage->slotclass);
 
+		if (!mfimage->filename) {
+			/* having no filename is valid for install hook only */
+			if (mfimage->hooks.install)
+				goto skip_filename_checks;
+			else
+				/* Should not be reached as the pre-conditions for optional 'filename' are already
+				 * checked during manifest parsing in manifest.c: parse_image() */
+				g_assert_not_reached();
+		}
+
+
 		/* if image filename is relative, make it absolute */
 		if (!g_path_is_absolute(mfimage->filename)) {
 			gchar *filename = g_build_filename(bundledir, mfimage->filename, NULL);
@@ -758,6 +802,7 @@ static gboolean pre_install_checks(gchar* bundledir, GList *install_images, GHas
 			return FALSE;
 		}
 
+skip_filename_checks:
 		if (!g_file_test(dest_slot->device, G_FILE_TEST_EXISTS)) {
 			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_NODST,
 					"Destination device '%s' not found", dest_slot->device);
@@ -895,17 +940,19 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 		g_free(slot_state->status);
 		slot_state->status = g_strdup("update");
 
-		g_message("Slot needs to be updated with %s", mfimage->filename);
-
 		r_context_end_step("check_slot", TRUE);
 
 		install_args_update(args, g_strdup_printf("Updating slot %s", dest_slot->name));
 
 		/* update slot */
-		if (mfimage->variant)
-			g_message("Updating %s with %s (variant: %s)", dest_slot->device, mfimage->filename, mfimage->variant);
-		else
-			g_message("Updating %s with %s", dest_slot->device, mfimage->filename);
+		if (mfimage->hooks.install) {
+			g_message("Updating %s with 'install' slot hook", dest_slot->device);
+		} else {
+			if (mfimage->variant)
+				g_message("Updating %s with %s (variant: %s)", dest_slot->device, mfimage->filename, mfimage->variant);
+			else
+				g_message("Updating %s with %s", dest_slot->device, mfimage->filename);
+		}
 
 		r_context_begin_step_formatted("copy_image", 0, "Copying image to %s", dest_slot->name);
 
@@ -1026,7 +1073,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 	// TODO: mount info in context ?
 	install_args_update(args, "Checking and mounting bundle...");
 
-	res = check_bundle(bundlefile, &bundle, TRUE, &ierror);
+	res = check_bundle(bundlefile, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
