@@ -28,7 +28,9 @@ GMainLoop *r_loop = NULL;
 int r_exit_status = 0;
 
 gboolean install_ignore_compatible, install_progressbar = FALSE;
+gboolean trust_environment = FALSE;
 gboolean verification_disabled = FALSE;
+gboolean no_check_time = FALSE;
 gboolean info_dumpcert = FALSE;
 gboolean status_detailed = FALSE;
 gchar *output_format = NULL;
@@ -36,6 +38,7 @@ gchar *signing_keyring = NULL;
 gchar *mksquashfs_args = NULL;
 gchar *casync_args = NULL;
 gchar *handler_args = NULL;
+gchar *bootslot = NULL;
 gboolean utf8_supported = FALSE;
 
 static gchar* make_progress_line(gint percentage)
@@ -464,6 +467,7 @@ out:
 G_GNUC_UNUSED
 static gboolean resign_start(int argc, char **argv)
 {
+	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
 	g_autoptr(RaucBundle) bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("resign start");
@@ -494,7 +498,12 @@ static gboolean resign_start(int argc, char **argv)
 		goto out;
 	}
 
-	if (!check_bundle(argv[2], &bundle, !verification_disabled, &ierror)) {
+	if (verification_disabled)
+		check_bundle_params |= CHECK_BUNDLE_NO_VERIFY;
+	if (no_check_time)
+		check_bundle_params |= CHECK_BUNDLE_NO_CHECK_TIME;
+
+	if (!check_bundle(argv[2], &bundle, check_bundle_params, &ierror)) {
 		g_printerr("%s\n", ierror->message);
 		g_clear_error(&ierror);
 		r_exit_status = 1;
@@ -512,6 +521,109 @@ out:
 	return TRUE;
 }
 
+static gboolean replace_signature_start(int argc, char **argv)
+{
+	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
+	RaucBundle *bundle = NULL;
+	GError *ierror = NULL;
+	g_debug("replace signature start");
+
+	if (argc < 3) {
+		g_printerr("An input bundle must be provided\n");
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (argc < 4) {
+		g_printerr("An input signature file must be provided\n");
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (argc < 5) {
+		g_printerr("An output bundle must be provided\n");
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (argc > 5) {
+		g_printerr("Excess argument: %s\n", argv[4]);
+		r_exit_status = 1;
+		goto out;
+	}
+
+	g_debug("input bundle: %s", argv[2]);
+	g_debug("input signature: %s", argv[3]);
+	g_debug("output file: %s", argv[4]);
+
+	if (verification_disabled)
+		check_bundle_params |= CHECK_BUNDLE_NO_VERIFY;
+	if (trust_environment)
+		check_bundle_params |= CHECK_BUNDLE_TRUST_ENV;
+
+	if (!check_bundle(argv[2], &bundle, check_bundle_params, &ierror)) {
+		g_printerr("%s\n", ierror->message);
+		g_clear_error(&ierror);
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (!replace_signature(bundle, argv[3], argv[4], check_bundle_params, &ierror)) {
+		g_printerr("Failed to replace signature: %s\n", ierror->message);
+		g_clear_error(&ierror);
+		r_exit_status = 1;
+		goto out;
+	}
+
+out:
+	return TRUE;
+}
+
+static gboolean extract_signature_start(int argc, char **argv)
+{
+	RaucBundle *bundle = NULL;
+	GError *ierror = NULL;
+	g_debug("extract signature start");
+
+	if (argc < 3) {
+		g_printerr("An input bundle must be provided\n");
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (argc < 4) {
+		g_printerr("An output signature file must be provided\n");
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (argc > 4) {
+		g_printerr("Excess argument: %s\n", argv[4]);
+		r_exit_status = 1;
+		goto out;
+	}
+
+	g_debug("input bundle: %s", argv[2]);
+	g_debug("output file: %s", argv[3]);
+
+	if (!check_bundle(argv[2], &bundle, CHECK_BUNDLE_DEFAULT, &ierror)) {
+		g_printerr("%s\n", ierror->message);
+		g_clear_error(&ierror);
+		r_exit_status = 1;
+		goto out;
+	}
+
+	if (!extract_signature(bundle, argv[3], &ierror)) {
+		g_printerr("Failed to extract signature: %s\n", ierror->message);
+		g_clear_error(&ierror);
+		r_exit_status = 1;
+		goto out;
+	}
+
+out:
+	return TRUE;
+}
+
 static gboolean extract_start(int argc, char **argv)
 {
 	RaucBundle *bundle = NULL;
@@ -539,7 +651,7 @@ static gboolean extract_start(int argc, char **argv)
 	g_debug("input bundle: %s", argv[2]);
 	g_debug("output dir: %s", argv[3]);
 
-	if (!check_bundle(argv[2], &bundle, TRUE, &ierror)) {
+	if (!check_bundle(argv[2], &bundle, CHECK_BUNDLE_DEFAULT, &ierror)) {
 		g_printerr("%s\n", ierror->message);
 		g_clear_error(&ierror);
 		r_exit_status = 1;
@@ -560,6 +672,7 @@ out:
 G_GNUC_UNUSED
 static gboolean convert_start(int argc, char **argv)
 {
+	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
 	RaucBundle *bundle = NULL;
 	GError *ierror = NULL;
 	g_debug("convert start");
@@ -592,7 +705,12 @@ static gboolean convert_start(int argc, char **argv)
 	g_debug("input bundle: %s", argv[2]);
 	g_debug("output bundle: %s", argv[3]);
 
-	if (!check_bundle(argv[2], &bundle, TRUE, &ierror)) {
+	if (verification_disabled)
+		check_bundle_params |= CHECK_BUNDLE_NO_VERIFY;
+	if (trust_environment)
+		check_bundle_params |= CHECK_BUNDLE_TRUST_ENV;
+
+	if (!check_bundle(argv[2], &bundle, check_bundle_params, &ierror)) {
 		g_printerr("%s\n", ierror->message);
 		g_clear_error(&ierror);
 		r_exit_status = 1;
@@ -612,6 +730,17 @@ out:
 	return TRUE;
 }
 
+/* Definition list for terminal colors */
+#define KNRM  "\x1B[0m"
+#define KRED  "\x1B[31m"
+#define KGRN  "\x1B[32m"
+#define KYEL  "\x1B[33m"
+#define KBLU  "\x1B[34m"
+#define KMAG  "\x1B[35m"
+#define KCYN  "\x1B[36m"
+#define KWHT  "\x1B[37m"
+#define KBLD  "\x1B[1m"
+
 /* Takes a shell variable and its desired argument as input and appends it to
  * the provided text with taking care of correct shell quoting */
 static void formatter_shell_append(GString* text, const gchar* varname, const gchar* argument)
@@ -639,7 +768,6 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 	formatter_shell_append(text, "RAUC_MF_DESCRIPTION", manifest->update_description);
 	formatter_shell_append(text, "RAUC_MF_BUILD", manifest->update_build);
 	g_string_append_printf(text, "RAUC_MF_IMAGES=%d\n", g_list_length(manifest->images));
-	g_string_append_printf(text, "RAUC_MF_FILES=%d\n", g_list_length(manifest->files));
 
 	hooks = g_ptr_array_new();
 	if (manifest->hooks.install_check == TRUE) {
@@ -682,17 +810,6 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 		cnt++;
 	}
 
-	cnt = 0;
-	for (GList *l = manifest->files; l != NULL; l = l->next) {
-		RaucFile *file = l->data;
-		g_string_append_printf(text, "RAUC_FILE_NAME_%d=%s\n", cnt, file->filename);
-		g_string_append_printf(text, "RAUC_FILE_CLASS_%d=%s\n", cnt, file->slotclass);
-		g_string_append_printf(text, "RAUC_FILE_DEST_%d=%s\n", cnt, file->destname);
-		g_string_append_printf(text, "RAUC_FILE_DIGEST_%d=%s\n", cnt, file->checksum.digest);
-		g_string_append_printf(text, "RAUC_FILE_SIZE_%d=%"G_GOFFSET_FORMAT "\n", cnt, file->checksum.size);
-		cnt++;
-	}
-
 	return g_string_free(text, FALSE);
 }
 
@@ -728,16 +845,20 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 	g_ptr_array_unref(hooks);
 
 	cnt = g_list_length(manifest->images);
-	g_string_append_printf(text, "%d Image%s%s\n", cnt, cnt == 1 ? "" : "s", cnt > 0 ? ":" : "");
+	g_string_append_printf(text, "\n%d Image%s%s\n", cnt, cnt == 1 ? "" : "s", cnt > 0 ? ":" : "");
 	cnt = 1;
 	for (GList *l = manifest->images; l != NULL; l = l->next) {
 		RaucImage *img = l->data;
-		g_string_append_printf(text, "(%d)\t%s\n", cnt, img->filename);
-		g_string_append_printf(text, "\tSlotclass: %s\n", img->slotclass);
+		g_string_append_printf(text, "  "KBLD "[%s]"KNRM "\n", img->slotclass);
 		if (img->variant)
 			g_string_append_printf(text, "\tVariant:   %s\n", img->variant);
-		g_string_append_printf(text, "\tChecksum:  %s\n", img->checksum.digest);
-		g_string_append_printf(text, "\tSize:      %"G_GOFFSET_FORMAT "\n", img->checksum.size);
+		if (img->filename) {
+			g_string_append_printf(text, "\tFilename:  %s\n", img->filename);
+			g_string_append_printf(text, "\tChecksum:  %s\n", img->checksum.digest);
+			g_string_append_printf(text, "\tSize:      %"G_GOFFSET_FORMAT "\n", img->checksum.size);
+		} else {
+			g_string_append_printf(text, "\t(no image file)\n");
+		}
 
 		hooks = g_ptr_array_new();
 		if (img->hooks.pre_install == TRUE) {
@@ -760,19 +881,6 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		cnt++;
 	}
 
-	cnt = g_list_length(manifest->files);
-	g_string_append_printf(text, "%d File%s%s\n", cnt, cnt == 1 ? "" : "s", cnt > 0 ? ":" : "");
-	cnt = 1;
-	for (GList *l = manifest->files; l != NULL; l = l->next) {
-		RaucFile *file = l->data;
-		g_string_append_printf(text, "(%d)\t%s\n", cnt, file->filename);
-		g_string_append_printf(text, "\tSlotclass: %s\n", file->slotclass);
-		g_string_append_printf(text, "\tDest:      %s\n", file->destname);
-		g_string_append_printf(text, "\tChecksum:  %s\n", file->checksum.digest);
-		g_string_append_printf(text, "\tSize:      %"G_GOFFSET_FORMAT "\n", file->checksum.size);
-		cnt++;
-	}
-
 	return g_string_free(text, FALSE);
 }
 
@@ -865,7 +973,6 @@ static gchar* info_formatter_json_pretty(RaucManifest *manifest)
 
 static gboolean info_start(int argc, char **argv)
 {
-	g_autofree gchar* tmpdir = NULL;
 	g_autofree gchar *bundlelocation = NULL;
 	g_autoptr(RaucManifest) manifest = NULL;
 	g_autoptr(RaucBundle) bundle = NULL;
@@ -873,6 +980,7 @@ static gboolean info_start(int argc, char **argv)
 	gboolean res = FALSE;
 	gchar* (*formatter)(RaucManifest *manifest) = NULL;
 	gchar *text;
+	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
 
 	if (argc < 3) {
 		g_printerr("A file name must be provided\n");
@@ -899,19 +1007,17 @@ static gboolean info_start(int argc, char **argv)
 		goto out;
 	}
 
-	tmpdir = g_dir_make_tmp("bundle-XXXXXX", &error);
-	if (!tmpdir) {
-		g_printerr("%s\n", error->message);
-		g_clear_error(&error);
-		goto out;
-	}
-
 	bundlelocation = resolve_bundle_path(argv[2]);
 	if (bundlelocation == NULL)
 		goto out;
 	g_debug("input bundle: %s", bundlelocation);
 
-	res = check_bundle(bundlelocation, &bundle, !verification_disabled, &error);
+	if (verification_disabled)
+		check_bundle_params |= CHECK_BUNDLE_NO_VERIFY;
+	if (no_check_time)
+		check_bundle_params |= CHECK_BUNDLE_NO_CHECK_TIME;
+
+	res = check_bundle(bundlelocation, &bundle, check_bundle_params, &error);
 	if (!res) {
 		g_printerr("%s\n", error->message);
 		g_clear_error(&error);
@@ -952,8 +1058,6 @@ static gboolean info_start(int argc, char **argv)
 
 out:
 	r_exit_status = res ? 0 : 1;
-	if (tmpdir)
-		rm_tree(tmpdir, NULL);
 	return TRUE;
 }
 
@@ -968,7 +1072,8 @@ typedef struct {
 
 static void free_status_print(RaucStatusPrint *status)
 {
-	g_return_if_fail(status);
+	if (!status)
+		return;
 
 	g_free(status->compatible);
 	g_free(status->variant);
@@ -977,22 +1082,10 @@ static void free_status_print(RaucStatusPrint *status)
 		g_hash_table_destroy(status->slots);
 
 	g_free(status);
-	return;
 }
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(RaucStatusPrint, free_status_print);
 
-/* Definition list for terminal colors */
-#define KNRM  "\x1B[0m"
-#define KRED  "\x1B[31m"
-#define KGRN  "\x1B[32m"
-#define KYEL  "\x1B[33m"
-#define KBLU  "\x1B[34m"
-#define KMAG  "\x1B[35m"
-#define KCYN  "\x1B[36m"
-#define KWHT  "\x1B[37m"
-#define KBLD  "\x1B[1m"
-
 static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint *status)
 {
 	RaucSlotStatus *slot_state = slot->status;
@@ -1437,7 +1530,7 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 		g_hash_table_insert(*slots, (gchar*)slot->name, slot);
 	}
 
-	/* Now we replace the dummy parent slots with the poitner to the real
+	/* Now we replace the dummy parent slots with the pointer to the real
 	 * parent slots */
 	g_hash_table_iter_init(&hiter, *slots);
 	while (g_hash_table_iter_next(&hiter, (gpointer*) &slot_name, (gpointer*) &iterslot)) {
@@ -1585,7 +1678,7 @@ static gboolean status_start(int argc, char **argv)
 
 		status_print->primary = r_boot_get_primary(&ierror);
 		if (!status_print->primary) {
-			g_printerr("Failed getting primary slot: %s\n", ierror->message);
+			g_printerr("%s\n", ierror->message);
 			g_clear_error(&ierror);
 		}
 
@@ -1613,7 +1706,7 @@ static gboolean status_start(int argc, char **argv)
 		slot_identifier = "booted";
 	} else if (argc == 4) {
 		slot_identifier = argv[3];
-	} else if (argc > 4) {
+	} else { /* argc > 4 */
 		g_warning("Too many arguments");
 		r_exit_status = 1;
 		goto out;
@@ -1672,6 +1765,59 @@ static gboolean service_start(int argc, char **argv)
 	return TRUE;
 }
 
+static gboolean mount_start(int argc, char **argv)
+{
+	g_autofree gchar *bundlelocation = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	GError *error = NULL;
+	gboolean res = FALSE;
+
+	if (argc < 3) {
+		g_printerr("A file name must be provided\n");
+		goto out;
+	}
+
+	if (argc > 3) {
+		g_printerr("Excess argument: %s\n", argv[3]);
+		goto out;
+	}
+
+	bundlelocation = resolve_bundle_path(argv[2]);
+	if (bundlelocation == NULL)
+		goto out; /* an error message was already printed by resolve_bundle_path */
+	g_debug("input bundle: %s", bundlelocation);
+
+	res = check_bundle(bundlelocation, &bundle, CHECK_BUNDLE_DEFAULT, &error);
+	if (!res) {
+		g_printerr("%s\n", error->message);
+		g_clear_error(&error);
+		goto out;
+	}
+
+	g_debug("bundle payload size: %"G_GOFFSET_FORMAT, bundle->size);
+
+	res = mount_bundle(bundle, &error);
+	if (!res) {
+		g_printerr("%s\n", error->message);
+		g_clear_error(&error);
+		goto out;
+	}
+
+	g_print("Mounted bundle at %s. Use 'umount %s' to unmount.\n", bundle->mount_point, bundle->mount_point);
+
+	/* The device mapper target and loopback devices are configured to remove
+	 * themselves on close. They are kept active by the mounted filesystem and
+	 * are automatically cleaned up when the user performs a normal unmount.
+	 * To avoid running the g_autoptr cleanup for the bundle, we use exit(0)
+	 * here.
+	 **/
+	exit(0);
+
+out:
+	r_exit_status = 1;
+	return FALSE;
+}
+
 static gboolean unknown_start(int argc, char **argv)
 {
 	g_debug("unknown start");
@@ -1684,12 +1830,15 @@ typedef enum  {
 	INSTALL,
 	BUNDLE,
 	RESIGN,
+	REPLACE_SIG,
+	EXTRACT_SIG,
 	EXTRACT,
 	CONVERT,
 	STATUS,
 	INFO,
 	WRITE_SLOT,
 	SERVICE,
+	MOUNT,
 } RaucCommandType;
 
 typedef struct {
@@ -1699,6 +1848,7 @@ typedef struct {
 	const gchar* summary;
 	gboolean (*cmd_handler)(int argc, char **argv);
 	GOptionGroup* options;
+	RContextConfigMode configmode;
 	gboolean while_busy;
 } RaucCommand;
 
@@ -1708,6 +1858,7 @@ static GOptionEntry entries_install[] = {
 	{"progress", '\0', 0, G_OPTION_ARG_NONE, &install_progressbar, "show progress bar", NULL},
 #else
 	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra handler arguments", "ARGS"},
+	{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
 #endif
 	{0}
 };
@@ -1719,12 +1870,22 @@ static GOptionEntry entries_bundle[] = {
 };
 
 static GOptionEntry entries_resign[] = {
+	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
+	{"no-check-time", '\0', 0, G_OPTION_ARG_NONE, &no_check_time, "don't check validity period of certificates against current time", NULL},
+	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
+	{0}
+};
+
+static GOptionEntry entries_replace[] = {
+	{"trust-environment", '\0', 0, G_OPTION_ARG_NONE, &trust_environment, "trust environment and skip bundle access checks", NULL},
 	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
 	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
 	{0}
 };
 
 static GOptionEntry entries_convert[] = {
+	{"trust-environment", '\0', 0, G_OPTION_ARG_NONE, &trust_environment, "trust environment and skip bundle access checks", NULL},
+	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
 	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
 	{"mksquashfs-args", '\0', 0, G_OPTION_ARG_STRING, &mksquashfs_args, "mksquashfs extra args", "ARGS"},
 	{"casync-args", '\0', 0, G_OPTION_ARG_STRING, &casync_args, "casync extra args", "ARGS"},
@@ -1733,6 +1894,7 @@ static GOptionEntry entries_convert[] = {
 
 static GOptionEntry entries_info[] = {
 	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
+	{"no-check-time", '\0', 0, G_OPTION_ARG_NONE, &no_check_time, "don't check validity period of certificates against current time", NULL},
 	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
 	{"dump-cert", '\0', 0, G_OPTION_ARG_NONE, &info_dumpcert, "dump certificate", NULL},
 	{0}
@@ -1741,17 +1903,22 @@ static GOptionEntry entries_info[] = {
 static GOptionEntry entries_status[] = {
 	{"detailed", '\0', 0, G_OPTION_ARG_NONE, &status_detailed, "show more status details", NULL},
 	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
+#if ENABLE_SERVICE == 0
+	{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
+#endif
 	{0}
 };
 
 static GOptionEntry entries_service[] = {
 	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra handler arguments", "ARGS"},
+	{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
 	{0}
 };
 
 static GOptionGroup *install_group;
 static GOptionGroup *bundle_group;
 static GOptionGroup *resign_group;
+static GOptionGroup *replace_group;
 static GOptionGroup *convert_group;
 static GOptionGroup *info_group;
 static GOptionGroup *status_group;
@@ -1769,6 +1936,9 @@ static void create_option_groups(void)
 		resign_group  = g_option_group_new("resign", "Resign options:", "help dummy", NULL, NULL);
 		g_option_group_add_entries(resign_group, entries_resign);
 
+		replace_group  = g_option_group_new("replace-signature", "Replace signature options:", "help dummy", NULL, NULL);
+		g_option_group_add_entries(replace_group, entries_replace);
+
 		convert_group = g_option_group_new("convert", "Convert options:", "help dummy", NULL, NULL);
 		g_option_group_add_entries(convert_group, entries_convert);
 	}
@@ -1786,8 +1956,7 @@ static void create_option_groups(void)
 static void cmdline_handler(int argc, char **argv)
 {
 	gboolean help = FALSE, debug = FALSE, version = FALSE;
-	gchar *confpath = NULL, *certpath = NULL, *keypath = NULL, *keyring = NULL, **intermediate = NULL, *mount = NULL,
-	      *bootslot = NULL;
+	gchar *confpath = NULL, *certpath = NULL, *keypath = NULL, *keyring = NULL, **intermediate = NULL, *mount = NULL;
 	char *cmdarg = NULL;
 	g_autoptr(GOptionContext) context = NULL;
 	GOptionEntry entries[] = {
@@ -1797,7 +1966,6 @@ static void cmdline_handler(int argc, char **argv)
 		{"keyring", '\0', 0, G_OPTION_ARG_FILENAME, &keyring, "keyring file", "PEMFILE"},
 		{"intermediate", '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &intermediate, "intermediate CA file name", "PEMFILE"},
 		{"mount", '\0', 0, G_OPTION_ARG_FILENAME, &mount, "mount prefix", "PATH"},
-		{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
 		{"debug", 'd', 0, G_OPTION_ARG_NONE, &debug, "enable debug output", NULL},
 		{"version", '\0', 0, G_OPTION_ARG_NONE, &version, "display version", NULL},
 		{"help", 'h', 0, G_OPTION_ARG_NONE, &help, NULL, NULL},
@@ -1808,26 +1976,53 @@ static void cmdline_handler(int argc, char **argv)
 	g_autofree gchar *text = NULL;
 
 	RaucCommand rcommands[] = {
-		{UNKNOWN, "help", "<COMMAND>", "Print help", unknown_start, NULL, TRUE},
-		{INSTALL, "install", "install <BUNDLE>", "Install a bundle", install_start, install_group, FALSE},
+		{UNKNOWN, "help", "<COMMAND>",
+		 "Print help",
+		 unknown_start, NULL, R_CONTEXT_CONFIG_MODE_NONE, TRUE},
+		{INSTALL, "install", "install <BUNDLE>",
+		 "Install a bundle",
+		 install_start, install_group, R_CONTEXT_CONFIG_MODE_REQUIRED, FALSE},
 #if ENABLE_CREATE == 1
-		{BUNDLE, "bundle", "bundle <INPUTDIR> <BUNDLENAME>", "Create a bundle from a content directory", bundle_start, bundle_group, FALSE},
-		{RESIGN, "resign", "resign <INBUNDLE> <OUTBUNDLE>", "Resign an already signed bundle", resign_start, resign_group, FALSE},
-		{CONVERT, "convert", "convert <INBUNDLE> <OUTBUNDLE>", "Convert to casync index bundle and store", convert_start, convert_group, FALSE},
+		{BUNDLE, "bundle", "bundle <INPUTDIR> <BUNDLENAME>",
+		 "Create a bundle from a content directory",
+		 bundle_start, bundle_group, R_CONTEXT_CONFIG_MODE_NONE, FALSE},
+		{RESIGN, "resign", "resign <INBUNDLE> <OUTBUNDLE>",
+		 "Resign an already signed bundle",
+		 resign_start, resign_group, R_CONTEXT_CONFIG_MODE_NONE, FALSE},
+		{CONVERT, "convert", "convert <INBUNDLE> <OUTBUNDLE>",
+		 "Convert to casync index bundle and store",
+		 convert_start, convert_group, R_CONTEXT_CONFIG_MODE_NONE, FALSE},
+		{REPLACE_SIG, "replace-signature", "replace-signature <INBUMDLE> <INPUTSIG> <OUTBUNDLE>",
+		 "Replaces the signature of an already signed bundle",
+		 replace_signature_start, replace_group, R_CONTEXT_CONFIG_MODE_NONE, FALSE},
+		{EXTRACT_SIG, "extract-signature", "extract-signature <BUNDLENAME> <OUTPUTSIG>",
+		 "Extract the bundle signature",
+		 extract_signature_start, NULL, R_CONTEXT_CONFIG_MODE_NONE, FALSE},
 #endif
-		{EXTRACT, "extract", "extract <BUNDLENAME> <OUTPUTDIR>", "Extract the bundle content", extract_start, NULL, FALSE},
-		{INFO, "info", "info <FILE>", "Print bundle info", info_start, info_group, FALSE},
+		{EXTRACT, "extract", "extract <BUNDLENAME> <OUTPUTDIR>",
+		 "Extract the bundle content",
+		 extract_start, NULL, R_CONTEXT_CONFIG_MODE_AUTO, FALSE},
+		{INFO, "info", "info <FILE>",
+		 "Print bundle info",
+		 info_start, info_group, R_CONTEXT_CONFIG_MODE_AUTO, FALSE},
 		{STATUS, "status", "status",
 		 "Show system status\n\n"
 		 "List of status commands (default slot is the currently booted slot):\n"
 		 "  mark-good [booted | other | <SLOT_NAME>] \tMark the slot as good\n"
 		 "  mark-bad [booted | other | <SLOT_NAME>] \tMark the slot as bad\n"
 		 "  mark-active [booted | other | <SLOT_NAME>] \tMark the slot as active",
-		 status_start, status_group, TRUE},
-		{WRITE_SLOT, "write-slot", "write-slot <SLOTNAME> <IMAGE>", "Write image to slot and bypass all update logic", write_slot_start, NULL, FALSE},
+		 status_start, status_group, R_CONTEXT_CONFIG_MODE_REQUIRED, TRUE},
+		{WRITE_SLOT, "write-slot", "write-slot <SLOTNAME> <IMAGE>",
+		 "Write image to slot and bypass all update logic",
+		 write_slot_start, NULL, R_CONTEXT_CONFIG_MODE_REQUIRED, FALSE},
 #if ENABLE_SERVICE == 1
-		{SERVICE, "service", "service", "Start RAUC service", service_start, service_group, TRUE},
+		{SERVICE, "service", "service",
+		 "Start RAUC service",
+		 service_start, service_group, R_CONTEXT_CONFIG_MODE_REQUIRED, TRUE},
 #endif
+		{MOUNT, "mount", "mount <BUNDLENAME>",
+		 "Mount a bundle (for development purposes)",
+		 mount_start, NULL, R_CONTEXT_CONFIG_MODE_REQUIRED, TRUE},
 		{0}
 	};
 	RaucCommand *rc;
@@ -1838,20 +2033,26 @@ static void cmdline_handler(int argc, char **argv)
 	g_option_context_set_ignore_unknown_options(context, TRUE);
 	g_option_context_add_main_entries(context, entries, NULL);
 	g_option_context_set_description(context,
+			"Command-specific help:\n"
+			"  rauc <COMMAND> --help\n"
+			"\n"
 			"List of rauc commands:\n"
 #if ENABLE_CREATE == 1
-			"  bundle\tCreate a bundle\n"
-			"  resign\tResign an already signed bundle\n"
-			"  convert\tConvert classic to casync bundle\n"
+			"  bundle\t\tCreate a bundle\n"
+			"  resign\t\tResign an already signed bundle\n"
+			"  convert\t\tConvert classic to casync bundle\n"
+			"  replace-signature\tReplaces the signature of an already signed bundle\n"
+			"  extract-signature\tExtract the bundle signature\n"
 #endif
-			"  extract\tExtract the bundle content\n"
-			"  install\tInstall a bundle\n"
-			"  info\t\tShow file information\n"
+			"  extract\t\tExtract the bundle content\n"
+			"  install\t\tInstall a bundle\n"
+			"  info\t\t\tShow file information\n"
+			"  mount\t\t\tMount a bundle\n"
 #if ENABLE_SERVICE == 1
-			"  service\tStart RAUC service\n"
+			"  service\t\tStart RAUC service\n"
 #endif
-			"  status\tShow status\n"
-			"  write-slot\tWrite image to slot and bypass all update logic\n"
+			"  status\t\tShow status\n"
+			"  write-slot\t\tWrite image to slot and bypass all update logic\n"
 			"\n"
 			"Environment variables:\n"
 			"  RAUC_PKCS11_MODULE  Library filename for PKCS#11 module (signing only)\n"
@@ -1899,8 +2100,6 @@ static void cmdline_handler(int argc, char **argv)
 		goto print_help;
 	}
 
-
-
 	/* try to get known command */
 	rc = rcommands;
 	while (rc->name) {
@@ -1943,6 +2142,13 @@ static void cmdline_handler(int argc, char **argv)
 	/* configuration updates are handled here */
 	if (!r_context_get_busy()) {
 		r_context_conf();
+		r_context_conf()->configmode = rcommand->configmode;
+		if (ENABLE_SERVICE) {
+			/* these commands are handled by the service and need no client config */
+			if (rcommand->type == INSTALL ||
+			    rcommand->type == STATUS)
+				r_context_conf()->configmode = R_CONTEXT_CONFIG_MODE_NONE;
+		}
 		if (confpath)
 			r_context_conf()->configpath = confpath;
 		if (certpath)
@@ -1981,6 +2187,13 @@ static void cmdline_handler(int argc, char **argv)
 		return;
 	}
 
+	if (!r_context_configure(&error)) {
+		g_printerr("Failed to initialize context: %s\n", error->message);
+		g_clear_error(&error);
+		r_exit_status = 1;
+		return;
+	}
+
 	/* real commands are handled here */
 	if (rcommand->cmd_handler) {
 		rcommand->cmd_handler(argc, argv);
