@@ -6,7 +6,6 @@
 #include "utils.h"
 
 #define RAUC_IMAGE_PREFIX	"image"
-#define RAUC_FILE_PREFIX	"file"
 
 #define R_MANIFEST_ERROR r_manifest_error_quark()
 GQuark r_manifest_error_quark(void)
@@ -19,7 +18,7 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 	g_autoptr(RaucImage) iimage = g_new0(RaucImage, 1);
 	g_auto(GStrv) groupsplit = NULL;
 	gchar *value;
-	gchar **hooks;
+	g_auto(GStrv) hooks = NULL;
 	gsize entries;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -46,14 +45,15 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 		iimage->checksum.digest = value;
 	}
 	iimage->checksum.size = g_key_file_get_uint64(key_file,
-			group, "size", NULL);
-	g_key_file_remove_key(key_file, group, "size", NULL);
-
-	iimage->filename = key_file_consume_string(key_file, group, "filename", &ierror);
-	if (iimage->filename == NULL) {
+			group, "size", &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+		iimage->checksum.size = -1;
+		g_clear_error(&ierror);
+	} else if (ierror) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
+	g_key_file_remove_key(key_file, group, "size", NULL);
 
 	hooks = g_key_file_get_string_list(key_file, group, "hooks", &entries, NULL);
 	for (gsize j = 0; j < entries; j++) {
@@ -64,12 +64,19 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 		} else if (g_strcmp0(hooks[j], "post-install") == 0) {
 			iimage->hooks.post_install = TRUE;
 		} else {
-			g_warning("hook key %s not supported", hooks[j]);
+			g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+					"slot hook type '%s' not supported", hooks[j]);
+			goto out;
 		}
 	}
 	g_key_file_remove_key(key_file, group, "hooks", NULL);
 
-	g_strfreev(hooks);
+	iimage->filename = key_file_consume_string(key_file, group, "filename", &ierror);
+	/* 'filename' is optional only for 'install' hooks */
+	if (iimage->filename == NULL && !iimage->hooks.install) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
 
 	if (!check_remaining_keys(key_file, group, &ierror)) {
 		g_propagate_error(error, ierror);
@@ -102,7 +109,7 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	g_autofree gchar *tmp = NULL;
 	gchar **groups;
 	gsize group_count;
-	gchar **bundle_hooks;
+	g_auto(GStrv) bundle_hooks = NULL;
 	gsize hook_entries;
 
 	g_assert_null(*manifest);
@@ -145,14 +152,6 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	}
 	g_key_file_remove_group(key_file, "bundle", NULL);
 
-	/* parse [keyring] section */
-	raucm->keyring = key_file_consume_string(key_file, "keyring", "archive", NULL);
-	if (!check_remaining_keys(key_file, "keyring", &ierror)) {
-		g_propagate_error(error, ierror);
-		goto free;
-	}
-	g_key_file_remove_group(key_file, "keyring", NULL);
-
 	/* parse [handler] section */
 	raucm->handler_name = key_file_consume_string(key_file, "handler", "filename", NULL);
 	raucm->handler_args = key_file_consume_string(key_file, "handler", "args", NULL);
@@ -170,10 +169,11 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 		if (g_strcmp0(bundle_hooks[j], "install-check") == 0) {
 			raucm->hooks.install_check = TRUE;
 		} else {
-			g_warning("hook key %s not supported", bundle_hooks[j]);
+			g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+					"install hook type '%s' not supported", bundle_hooks[j]);
+			goto free;
 		}
 	}
-	g_strfreev(bundle_hooks);
 
 	if (!check_remaining_keys(key_file, "hooks", &ierror)) {
 		g_propagate_error(error, ierror);
@@ -181,7 +181,7 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	}
 	g_key_file_remove_group(key_file, "hooks", NULL);
 
-	/* parse [image.<slotclass>] and [file.<slotclass>/<destname>] sections */
+	/* parse [image.<slotclass>] sections */
 	groups = g_key_file_get_groups(key_file, &group_count);
 	for (gsize i = 0; i < group_count; i++) {
 		if (g_str_has_prefix(groups[i], RAUC_IMAGE_PREFIX ".")) {
@@ -193,51 +193,6 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 			}
 
 			raucm->images = g_list_append(raucm->images, image);
-		} else if (g_str_has_prefix(groups[i], RAUC_FILE_PREFIX ".")) {
-			RaucFile *file;
-			gchar *value;
-			gchar **groupsplit = g_strsplit(groups[i], ".", 2);
-			gchar **destsplit = NULL;
-			if (groupsplit == NULL || g_strv_length(groupsplit) < 2) {
-				g_strfreev(groupsplit);
-				continue;
-			}
-			destsplit = g_strsplit(groupsplit[1], "/", 2);
-			g_strfreev(groupsplit);
-
-			if (destsplit == NULL || g_strv_length(destsplit) < 2) {
-				g_strfreev(destsplit);
-				continue;
-			}
-
-			file = g_new0(RaucFile, 1);
-
-			file->slotclass = g_strdup(destsplit[0]);
-			file->destname = g_strdup(destsplit[1]);
-
-			value = key_file_consume_string(key_file, groups[i], "sha256", NULL);
-			if (value) {
-				file->checksum.type = G_CHECKSUM_SHA256;
-				file->checksum.digest = value;
-			}
-			file->checksum.size = g_key_file_get_uint64(key_file,
-					groups[i], "size", NULL);
-			g_key_file_remove_key(key_file, groups[i], "size", NULL);
-
-
-			file->filename = key_file_consume_string(key_file, groups[i], "filename", &ierror);
-			if (file->filename == NULL) {
-				g_propagate_error(error, ierror);
-				goto free;
-			}
-
-			raucm->files = g_list_append(raucm->files, file);
-
-			if (!check_remaining_keys(key_file, groups[i], &ierror)) {
-				g_propagate_error(error, ierror);
-				goto free;
-			}
-			g_key_file_remove_group(key_file, groups[i], NULL);
 		}
 	}
 
@@ -312,6 +267,7 @@ out:
 
 static gboolean check_manifest_common(const RaucManifest *mf, GError **error)
 {
+	gboolean have_hooks = FALSE;
 	gboolean res = FALSE;
 
 	switch (mf->bundle_format) {
@@ -329,7 +285,12 @@ static gboolean check_manifest_common(const RaucManifest *mf, GError **error)
 		RaucImage *image = l->data;
 
 		g_assert(image);
-		g_assert(image->filename);
+
+		/* Having no 'filename' set is valid for 'install' hook only.
+		 * This is already ensured during manifest parsing, thus simply
+		 * skip further checks here */
+		if (!image->filename)
+			continue;
 
 		if (image->checksum.type != G_CHECKSUM_SHA256) {
 			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Unsupported checksum algorithm for image %s", image->filename);
@@ -339,12 +300,48 @@ static gboolean check_manifest_common(const RaucManifest *mf, GError **error)
 			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing digest for image %s", image->filename);
 			goto out;
 		}
-		if (!image->checksum.size) {
-			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing size for image %s", image->filename);
-			goto out;
+		if (image->checksum.size < 0) {
+			/* RAUC versions before v1.5 allowed zero-size images but did not handle this explicitly.
+			 * Thus, bundles created did have a valid 'filename=' manifest entry
+			 * but the 'size=' entry was considered as empty and not set at all.
+			 * Retain support for this case, at least for the 'install' per-slot hook use-case
+			 * where an image file can be optional. */
+			if (image->hooks.install) {
+				g_message("Missing size parameter for image '%s'", image->filename);
+				image->checksum.size = 0;
+			} else {
+				g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Missing size for image %s", image->filename);
+				goto out;
+			}
 		}
 	}
 
+	/* Check for hook file set if hooks are enabled */
+
+	if (mf->hooks.install_check == TRUE)
+		have_hooks = TRUE;
+
+	for (GList *l = mf->images; l != NULL; l = l->next) {
+		RaucImage *image = l->data;
+		if (image->hooks.pre_install == TRUE) {
+			have_hooks = TRUE;
+			break;
+		}
+		if (image->hooks.install == TRUE) {
+			have_hooks = TRUE;
+			break;
+		}
+		if (image->hooks.post_install == TRUE) {
+			have_hooks = TRUE;
+			break;
+		}
+	}
+
+	if (have_hooks && !mf->hook_name) {
+		g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_CHECK_ERROR, "Hooks used, but no hook 'filename' defined in [hooks] section");
+		goto out;
+	}
+
 	res = TRUE;
 out:
 	return res;
@@ -357,8 +354,7 @@ gboolean check_manifest_internal(const RaucManifest *mf, GError **error)
 
 	r_context_begin_step("check_manifest", "Checking manifest contents", 0);
 
-	res = check_manifest_common(mf, &ierror);
-	if (!res) {
+	if (!check_manifest_common(mf, &ierror)) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
@@ -401,8 +397,7 @@ gboolean check_manifest_external(const RaucManifest *mf, GError **error)
 
 	r_context_begin_step("check_manifest", "Checking manifest contents", 0);
 
-	res = check_manifest_common(mf, &ierror);
-	if (!res) {
+	if (!check_manifest_common(mf, &ierror)) {
 		g_propagate_error(error, ierror);
 		goto out;
 	}
@@ -498,9 +493,6 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 			break;
 	}
 
-	if (mf->keyring)
-		g_key_file_set_string(key_file, "keyring", "archive", mf->keyring);
-
 	if (mf->handler_name)
 		g_key_file_set_string(key_file, "handler", "filename", mf->handler_name);
 
@@ -538,7 +530,7 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 
 		if (image->checksum.type == G_CHECKSUM_SHA256)
 			g_key_file_set_string(key_file, group, "sha256", image->checksum.digest);
-		if (image->checksum.size)
+		if (image->checksum.size >= 0)
 			g_key_file_set_uint64(key_file, group, "size", image->checksum.size);
 
 		if (image->filename)
@@ -561,24 +553,6 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 		}
 	}
 
-	for (GList *l = mf->files; l != NULL; l = l->next) {
-		RaucFile *file = l->data;
-		g_autofree gchar *group = NULL;
-
-		if (!file || !file->slotclass || !file->destname)
-			continue;
-
-		group = g_strconcat(RAUC_FILE_PREFIX ".", file->slotclass, "/", file->destname, NULL);
-
-		if (file->checksum.type == G_CHECKSUM_SHA256)
-			g_key_file_set_string(key_file, group, "sha256", file->checksum.digest);
-		if (file->checksum.size)
-			g_key_file_set_uint64(key_file, group, "size", file->checksum.size);
-
-		if (file->filename)
-			g_key_file_set_string(key_file, group, "filename", file->filename);
-	}
-
 	return g_steal_pointer(&key_file);
 }
 
@@ -622,7 +596,8 @@ void r_free_image(gpointer data)
 {
 	RaucImage *image = (RaucImage*) data;
 
-	g_return_if_fail(image);
+	if (!image)
+		return;
 
 	g_free(image->slotclass);
 	g_free(image->variant);
@@ -631,22 +606,10 @@ void r_free_image(gpointer data)
 	g_free(image);
 }
 
-void r_free_file(gpointer data)
-{
-	RaucFile *file = (RaucFile*) data;
-
-	g_return_if_fail(file);
-
-	g_free(file->slotclass);
-	g_free(file->destname);
-	g_free(file->checksum.digest);
-	g_free(file->filename);
-	g_free(file);
-}
-
 void free_manifest(RaucManifest *manifest)
 {
-	g_return_if_fail(manifest);
+	if (!manifest)
+		return;
 
 	g_free(manifest->update_compatible);
 	g_free(manifest->update_version);
@@ -654,16 +617,24 @@ void free_manifest(RaucManifest *manifest)
 	g_free(manifest->update_build);
 	g_free(manifest->bundle_verity_hash);
 	g_free(manifest->bundle_verity_salt);
-	g_free(manifest->keyring);
 	g_free(manifest->handler_name);
 	g_free(manifest->handler_args);
 	g_free(manifest->hook_name);
 	g_list_free_full(manifest->images, r_free_image);
-	g_list_free_full(manifest->files, r_free_file);
 	g_free(manifest);
 }
 
-gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *dir, GError **error)
+/**
+ * Updates checksums for images listed in the manifest and found in
+ * the bundle directory.
+ *
+ * @param manifest pointer to the manifest
+ * @param dir Directory with the bundle content
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE on success, FALSE if an error occurred
+ */
+static gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *dir, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = TRUE;
@@ -671,20 +642,16 @@ gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *dir, GEr
 
 	for (GList *elem = manifest->images; elem != NULL; elem = elem->next) {
 		RaucImage *image = elem->data;
-		g_autofree gchar *filename = g_build_filename(dir, image->filename, NULL);
-		res = compute_checksum(&image->checksum, filename, &ierror);
-		if (!res) {
-			g_warning("Failed updating checksum: %s", ierror->message);
-			g_clear_error(&ierror);
-			had_errors = TRUE;
-			break;
-		}
-	}
+		g_autofree gchar *filename = NULL;
 
-	for (GList *elem = manifest->files; elem != NULL; elem = elem->next) {
-		RaucFile *file = elem->data;
-		g_autofree gchar *filename = g_build_filename(dir, file->filename, NULL);
-		res = compute_checksum(&file->checksum, filename, &ierror);
+		/* If no filename is set (valid for 'install' hook) explicitly set size to -1 */
+		if (!image->filename) {
+			image->checksum.size = -1;
+			continue;
+		}
+
+		filename = g_build_filename(dir, image->filename, NULL);
+		res = compute_checksum(&image->checksum, filename, &ierror);
 		if (!res) {
 			g_warning("Failed updating checksum: %s", ierror->message);
 			g_clear_error(&ierror);
@@ -700,3 +667,32 @@ gboolean update_manifest_checksums(RaucManifest *manifest, const gchar *dir, GEr
 
 	return res;
 }
+
+gboolean sync_manifest_with_contentdir(RaucManifest *manifest, const gchar *dir, GError **error)
+{
+	g_return_val_if_fail(manifest, FALSE);
+	g_return_val_if_fail(dir, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	/* Check for missing image files */
+	for (GList *elem = manifest->images; elem != NULL; elem = elem->next) {
+		RaucImage *image = elem->data;
+		g_autofree gchar *filename = g_build_filename(dir, image->filename, NULL);
+		if (!g_file_test(filename, G_FILE_TEST_EXISTS)) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_ERROR_CHECKSUM, "image file '%s' for slot '%s' does not exist in bundle content dir (%s)", image->filename, image->slotclass, dir);
+			return FALSE;
+		}
+	}
+
+	/* Check for missing hook file */
+	if (manifest->hook_name) {
+		g_autofree gchar *hookpath = NULL;
+		hookpath = g_build_filename(dir, manifest->hook_name, NULL);
+		if (!g_file_test(hookpath, G_FILE_TEST_EXISTS)) {
+			g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_ERROR_CHECKSUM, "hook file '%s' does not exist in bundle content dir (%s)", manifest->hook_name, dir);
+			return FALSE;
+		}
+	}
+
+	return update_manifest_checksums(manifest, dir, error);
+}
