@@ -4,7 +4,8 @@ void r_slot_free(gpointer value)
 {
 	RaucSlot *slot = (RaucSlot*)value;
 
-	g_return_if_fail(slot);
+	if (!slot)
+		return;
 
 	g_free(slot->description);
 	g_free(slot->device);
@@ -20,7 +21,8 @@ void r_slot_free(gpointer value)
 
 void r_slot_free_status(RaucSlotStatus *slotstatus)
 {
-	g_return_if_fail(slotstatus);
+	if (!slotstatus)
+		return;
 
 	g_free(slotstatus->bundle_compatible);
 	g_free(slotstatus->bundle_version);
@@ -123,10 +125,25 @@ RaucSlotType supported_slot_types[] = {
 	{"ubifs", TRUE},
 	{"ubivol", FALSE},
 	{"nand", FALSE},
+	{"nor", FALSE},
+	{"boot-emmc", FALSE},
+	{"boot-mbr-switch", FALSE},
+	{"boot-gpt-switch", FALSE},
 	{"vfat", TRUE},
 	{}
 };
 
+gboolean r_slot_is_valid_type(const gchar *type)
+{
+	for (RaucSlotType *slot_type = supported_slot_types; slot_type->name != NULL; slot_type++) {
+		if (g_strcmp0(type, slot_type->name) == 0) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 gboolean r_slot_is_mountable(RaucSlot *slot)
 {
 	for (RaucSlotType *slot_type = supported_slot_types; slot_type->name != NULL; slot_type++) {
