@@ -196,11 +196,8 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 		return FALSE;
 	}
 
-	/* Do not close fd automatically to give us the chance to call fsync() on it before closing */
-	g_unix_output_stream_set_close_fd(outstream, FALSE);
-
 	size = g_output_stream_splice(G_OUTPUT_STREAM(outstream), instream,
-			G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+			G_OUTPUT_STREAM_SPLICE_NONE,
 			NULL,
 			&ierror);
 	if (size == -1) {
@@ -224,16 +221,10 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 
 	/* flush to block device before closing to assure content is written to disk */
 	if (fsync(out_fd) == -1) {
-		close(out_fd); /* Silent attempt to close as we failed, anyway */
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED, "Syncing content to disk failed: %s", strerror(errno));
 		return FALSE;
 	}
 
-	if (close(out_fd) == -1) {
-		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED, "Closing output device failed: %s", strerror(errno));
-		return FALSE;
-	}
-
 	return TRUE;
 }
 
@@ -410,6 +401,12 @@ static gboolean copy_raw_image_to_dev(RaucImage *image, RaucSlot *slot, GError *
 		goto out;
 	}
 
+	res = g_output_stream_close(G_OUTPUT_STREAM(outstream), NULL, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 out:
 	return res;
 }
@@ -547,6 +544,51 @@ out:
 	return res;
 }
 
+/**
+ * Create valid / safe vfat label names.
+ *
+ * mkfs.vfat label requirements:
+ *
+ * | mkfs.fat: Warning: lowercase labels might not work properly on some systems
+ * | mkfs.vfat: Labels with characters *?.,;:/\|+=<>[]" are not allowed
+ * | mkfs.vfat: Label can be no longer than 11 characters
+ *
+ * This cuts input at 11 characters, makes all characters uppercase and
+ * replaces all invalid characters by '_' (underscore)
+ *
+ * @param name input name to create vfat label from
+ * @return newly-allocated string to be used as "-n" argument for mkfs.vfat
+ */
+static gchar* vfat_label_generator(const gchar *name)
+{
+	gchar *label_name;
+	const gchar *invalid_chars = "*?.,;:/\\|+=<>[]\"";
+
+	g_return_val_if_fail(name, NULL);
+
+	/* limit label length to 11 characters */
+	if (strlen(name) > 11)
+		label_name = g_strndup(name, 11);
+	else
+		label_name = g_strdup(name);
+
+	for (gchar *c = label_name; *c != '\0'; c++) {
+		/* make chars uppercase */
+		if (g_ascii_islower(*c))
+			*c = g_ascii_toupper(*c);
+		/* replace invalid chars */
+		for (size_t i = 0; i < strlen(invalid_chars); i++) {
+			g_message("r: %c", invalid_chars[i]);
+			if (*c == invalid_chars[i]) {
+				*c = '_';
+				break;
+			}
+		}
+	}
+
+	return label_name;
+}
+
 static gboolean vfat_format_slot(RaucSlot *dest_slot, GError **error)
 {
 	g_autoptr(GSubprocess) sproc = NULL;
@@ -555,10 +597,8 @@ static gboolean vfat_format_slot(RaucSlot *dest_slot, GError **error)
 	g_autoptr(GPtrArray) args = g_ptr_array_new_full(4, g_free);
 
 	g_ptr_array_add(args, g_strdup("mkfs.vfat"));
-	if (strlen(dest_slot->name) <= 16) {
-		g_ptr_array_add(args, g_strdup("-n"));
-		g_ptr_array_add(args, g_strdup(dest_slot->name));
-	}
+	g_ptr_array_add(args, g_strdup("-n"));
+	g_ptr_array_add(args, vfat_label_generator(dest_slot->name));
 	g_ptr_array_add(args, g_strdup(dest_slot->device));
 	g_ptr_array_add(args, NULL);
 
@@ -584,7 +624,41 @@ out:
 	return res;
 }
 
-static gboolean nand_format_slot(const gchar *device, GError **error)
+static gboolean nor_write_slot(const gchar *image, const gchar *device, GError **error)
+{
+	g_autoptr(GSubprocess) sproc = NULL;
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(5, g_free);
+
+	g_ptr_array_add(args, g_strdup("flashcp"));
+	g_ptr_array_add(args, g_strdup(image));
+	g_ptr_array_add(args, g_strdup(device));
+	g_ptr_array_add(args, NULL);
+
+	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_NONE, &ierror);
+	if (sproc == NULL) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to start flashcp: ");
+		goto out;
+	}
+
+	res = g_subprocess_wait_check(sproc, NULL, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to run flashcp: ");
+		goto out;
+	}
+
+out:
+	return res;
+}
+
+static gboolean flash_format_slot(const gchar *device, GError **error)
 {
 	g_autoptr(GSubprocess) sproc = NULL;
 	GError *ierror = NULL;
@@ -704,7 +778,7 @@ static gboolean unpack_archive(RaucImage *image, gchar *dest, GError **error)
 }
 
 /**
- * Executes the per-slot hook script.
+ * Executes the per-slot hook script with extra environment variables.
  *
  * @param hook_name file name of the hook script
  * @param hook_cmd first argument to the hook script
@@ -714,10 +788,11 @@ static gboolean unpack_archive(RaucImage *image, gchar *dest, GError **error)
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, RaucImage *image, RaucSlot *slot, GError **error)
+static gboolean run_slot_hook_extra_env(const gchar *hook_name, const gchar *hook_cmd, RaucImage *image, RaucSlot *slot, GHashTable *variables, GError **error)
 {
 	g_autoptr(GSubprocessLauncher) launcher = NULL;
 	g_autoptr(GSubprocess) sproc = NULL;
+	g_autofree gchar* image_size = NULL;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
 	RaucBundle *bundle;
@@ -726,7 +801,7 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 	g_assert_nonnull(slot->name);
 	g_assert_nonnull(slot->sclass);
 
-	g_message("Running slot hook %s for %s", hook_cmd, slot->name);
+	g_message("Running slot hook '%s' for %s", hook_cmd, slot->name);
 
 	launcher = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_NONE);
 
@@ -750,8 +825,10 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 		g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_MOUNT_POINT", slot->ext_mount_point, TRUE);
 	}
 	if (image) {
-		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_NAME", image->filename, TRUE);
-		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_DIGEST", image->checksum.digest, TRUE);
+		image_size = g_strdup_printf("%" G_GOFFSET_FORMAT, image->checksum.size);
+		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_NAME", image->filename ? image->filename : "", TRUE);
+		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_SIZE", image_size, TRUE);
+		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_DIGEST", image->checksum.digest ? image->checksum.digest : "", TRUE);
 		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_CLASS", image->slotclass, TRUE);
 	}
 	g_subprocess_launcher_setenv(launcher, "RAUC_MOUNT_PREFIX", r_context()->config->mount_prefix, TRUE);
@@ -771,6 +848,19 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 		g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_MOUNT_POINT", bundle->mount_point, TRUE);
 	}
 
+	if (variables) {
+		GHashTableIter iter;
+		gchar *key = NULL;
+		gchar *value = NULL;
+
+		/* copy the variables from the hashtable and add them to the
+		   subprocess environment */
+		g_hash_table_iter_init(&iter, variables);
+		while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
+			g_subprocess_launcher_setenv(launcher, g_strdup(key), g_strdup(value), TRUE);
+		}
+	}
+
 	sproc = g_subprocess_launcher_spawn(
 			launcher, &ierror,
 			hook_name,
@@ -797,7 +887,23 @@ out:
 	return res;
 }
 
-static gboolean mount_and_run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, RaucSlot *slot, GError **error)
+/**
+ * Executes the per-slot hook script without setting extra environment variables.
+ *
+ * @param hook_name file name of the hook script
+ * @param hook_cmd first argument to the hook script
+ * @param image image to be installed (optional)
+ * @param slot target slot
+ * @param error return location for a GError, or NULL
+ *
+ * @return TRUE on success, FALSE if an error occurred
+ */
+static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, RaucImage *image, RaucSlot *slot, GError **error)
+{
+	return run_slot_hook_extra_env(hook_name, hook_cmd, image, slot, NULL, error);
+}
+
+static gboolean mount_and_run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, RaucImage *image, RaucSlot *slot, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -816,7 +922,7 @@ static gboolean mount_and_run_slot_hook(const gchar *hook_name, const gchar *hoo
 
 	/* run slot install hook */
 	g_message("Running slot '%s' hook for %s", hook_cmd, slot->name);
-	res = run_slot_hook(hook_name, hook_cmd, NULL, slot, &ierror);
+	res = run_slot_hook(hook_name, hook_cmd, image, slot, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 	}
@@ -848,7 +954,7 @@ static gboolean img_to_ubivol_handler(RaucImage *image, RaucSlot *dest_slot, con
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -889,9 +995,15 @@ static gboolean img_to_ubivol_handler(RaucImage *image, RaucSlot *dest_slot, con
 		}
 	}
 
+	res = g_output_stream_close(G_OUTPUT_STREAM(outstream), NULL, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -911,7 +1023,7 @@ static gboolean img_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot, cons
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, dest_slot, &ierror);
+		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -952,9 +1064,15 @@ static gboolean img_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot, cons
 		}
 	}
 
+	res = g_output_stream_close(G_OUTPUT_STREAM(outstream), NULL, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, dest_slot, &ierror);
+		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -974,7 +1092,7 @@ static gboolean archive_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot,
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, dest_slot, &ierror);
+		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1007,7 +1125,7 @@ static gboolean archive_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot,
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto unmount_out;
@@ -1033,6 +1151,74 @@ out:
 	return res;
 }
 
+static gboolean archive_to_jffs2_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
+{
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	/* run slot pre install hook if enabled */
+	if (hook_name && image->hooks.pre_install) {
+		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+	/* erase */
+	g_message("Erasing slot mtd device %s", dest_slot->device);
+	res = flash_format_slot(dest_slot->device, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	/* jffs2 needs no formatting - mount directly */
+	g_message("Mounting jffs2 slot %s", dest_slot->device);
+	res = r_mount_slot(dest_slot, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	/* extract tar into mounted jffs2 volume */
+	g_message("Extracting %s to %s", image->filename, dest_slot->mount_point);
+	res = unpack_archive(image, dest_slot->mount_point, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto unmount_out;
+	}
+
+	/* run slot post install hook if enabled */
+	if (hook_name && image->hooks.post_install) {
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto unmount_out;
+		}
+	}
+
+unmount_out:
+	/* finally umount jffs2 volume */
+	g_message("Unmounting jffs2 slot %s", dest_slot->device);
+	ierror = NULL; /* any previous error was propagated already */
+	if (!r_umount_slot(dest_slot, &ierror)) {
+		res = FALSE;
+		if (error && *error) {
+			/* the previous error is more relevant here */
+			g_warning("Ignoring umount error after previous error: %s", ierror->message);
+			g_clear_error(&ierror);
+		} else {
+			g_propagate_error(error, ierror);
+		}
+	}
+
+out:
+	return res;
+}
+
 static gboolean archive_to_ext4_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
 	GError *ierror = NULL;
@@ -1042,7 +1228,7 @@ static gboolean archive_to_ext4_handler(RaucImage *image, RaucSlot *dest_slot, c
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1075,7 +1261,7 @@ static gboolean archive_to_ext4_handler(RaucImage *image, RaucSlot *dest_slot, c
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto unmount_out;
@@ -1110,7 +1296,7 @@ static gboolean archive_to_vfat_handler(RaucImage *image, RaucSlot *dest_slot, c
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1143,7 +1329,7 @@ static gboolean archive_to_vfat_handler(RaucImage *image, RaucSlot *dest_slot, c
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto unmount_out;
@@ -1169,14 +1355,14 @@ out:
 	return res;
 }
 
-static gboolean img_to_nand_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
+static gboolean img_to_nor_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1185,7 +1371,50 @@ static gboolean img_to_nand_handler(RaucImage *image, RaucSlot *dest_slot, const
 
 	/* erase */
 	g_message("erasing slot device %s", dest_slot->device);
-	res = nand_format_slot(dest_slot->device, &ierror);
+	res = flash_format_slot(dest_slot->device, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	/* write */
+	g_message("writing slot device %s", dest_slot->device);
+	res = nor_write_slot(image->filename, dest_slot->device, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	/* run slot post install hook if enabled */
+	if (hook_name && image->hooks.post_install) {
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+out:
+	return res;
+}
+
+static gboolean img_to_nand_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
+{
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+
+	/* run slot pre install hook if enabled */
+	if (hook_name && image->hooks.pre_install) {
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+	/* erase */
+	g_message("erasing slot device %s", dest_slot->device);
+	res = flash_format_slot(dest_slot->device, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1201,7 +1430,7 @@ static gboolean img_to_nand_handler(RaucImage *image, RaucSlot *dest_slot, const
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1219,7 +1448,7 @@ static gboolean img_to_fs_handler(RaucImage *image, RaucSlot *dest_slot, const g
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, dest_slot, &ierror);
+		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1244,7 +1473,7 @@ static gboolean img_to_fs_handler(RaucImage *image, RaucSlot *dest_slot, const g
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, dest_slot, &ierror);
+		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1258,20 +1487,11 @@ out:
 static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
 	gboolean res = FALSE;
-	int out_fd = -1, inactive_part;
+	int out_fd = -1, inactive_half;
 	g_autoptr(GUnixOutputStream) outstream = NULL;
 	GError *ierror = NULL;
 	struct boot_switch_partition dest_partition;
-
-	/* run slot pre install hook if enabled */
-	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL,
-				dest_slot, &ierror);
-		if (!res) {
-			g_propagate_error(error, ierror);
-			goto out;
-		}
-	}
+	g_autoptr(GHashTable) vars = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
 	res = r_mbr_switch_get_inactive_partition(dest_slot->device,
 			&dest_partition, dest_slot->region_start,
@@ -1282,12 +1502,12 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 	}
 
 	if (dest_partition.start == dest_slot->region_start)
-		inactive_part = 0;
+		inactive_half = 0;
 	else
-		inactive_part = 1;
+		inactive_half = 1;
 
-	g_message("Found inactive boot partition %d (pos. %"G_GUINT64_FORMAT "B, size %"G_GUINT64_FORMAT "B)",
-			inactive_part, dest_partition.start, dest_partition.size);
+	g_message("Found inactive (%s) half of boot partition region (pos. %"G_GUINT64_FORMAT "B, size %"G_GUINT64_FORMAT "B)",
+			inactive_half == 0 ? "first" : "second", dest_partition.start, dest_partition.size);
 
 	if (dest_partition.size < (guint64)image->checksum.size) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
@@ -1297,17 +1517,34 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Clearing inactive boot partition %d on %s", inactive_part,
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_ACTIVATING"),
+			g_strdup_printf("%d", inactive_half));
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_START"),
+			g_strdup_printf("%"G_GUINT64_FORMAT, dest_partition.start));
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_SIZE"),
+			g_strdup_printf("%"G_GUINT64_FORMAT, dest_partition.size));
+
+	/* run slot pre install hook if enabled */
+	if (hook_name && image->hooks.pre_install) {
+		res = run_slot_hook_extra_env(hook_name, R_SLOT_HOOK_PRE_INSTALL, image,
+				dest_slot, vars, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+	g_message("Clearing inactive (%s) half of boot partition region on %s", inactive_half == 0 ? "first" : "second",
 			dest_slot->device);
 
 	res = clear_boot_switch_partition(dest_slot->device, &dest_partition, &ierror);
 	if (!res) {
 		g_propagate_prefixed_error(error, ierror,
-				"Failed to clear inactive partition: ");
+				"Failed to clear inactive region: ");
 		goto out;
 	}
 
-	g_message("Opening inactive boot partition %d on %s", inactive_part,
+	g_message("Opening inactive (%s) half of boot partition region on %s", inactive_half == 0 ? "first" : "second",
 			dest_slot->device);
 
 	out_fd = open(dest_slot->device, O_WRONLY);
@@ -1336,8 +1573,8 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Copying image to inactive boot partition %d on %s",
-			inactive_part, dest_slot->device);
+	g_message("Copying image to inactive (%s) half of boot partition region on %s",
+			inactive_half == 0 ? "first" : "second", dest_slot->device);
 
 	res = copy_raw_image(image, outstream, &ierror);
 	if (!res) {
@@ -1345,9 +1582,7 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Setting MBR to switch boot partition");
-
-	res = r_mbr_switch_set_boot_partition(dest_slot->device, &dest_partition, &ierror);
+	res = g_output_stream_close(G_OUTPUT_STREAM(outstream), NULL, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1355,13 +1590,22 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL,
-				dest_slot, &ierror);
+		res = run_slot_hook_extra_env(hook_name, R_SLOT_HOOK_POST_INSTALL, image,
+				dest_slot, vars, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
 		}
 	}
+
+	g_message("Setting %s half of boot partition region active in MBR", inactive_half == 0 ? "first" : "second");
+
+	res = r_mbr_switch_set_boot_partition(dest_slot->device, &dest_partition, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 out:
 	if (out_fd >= 0)
 		close(out_fd);
@@ -1373,20 +1617,11 @@ G_GNUC_UNUSED
 static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
 	gboolean res = FALSE;
-	int out_fd = -1, inactive_part;
+	int out_fd = -1, inactive_half;
 	g_autoptr(GUnixOutputStream) outstream = NULL;
 	GError *ierror = NULL;
 	struct boot_switch_partition dest_partition;
-
-	/* run slot pre install hook if enabled */
-	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL,
-				dest_slot, &ierror);
-		if (!res) {
-			g_propagate_error(error, ierror);
-			goto out;
-		}
-	}
+	g_autoptr(GHashTable) vars = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
 	res = r_gpt_switch_get_inactive_partition(dest_slot->device,
 			&dest_partition, dest_slot->region_start,
@@ -1397,12 +1632,12 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 	}
 
 	if (dest_partition.start == dest_slot->region_start)
-		inactive_part = 0;
+		inactive_half = 0;
 	else
-		inactive_part = 1;
+		inactive_half = 1;
 
-	g_message("Found inactive boot partition %d (pos. %"G_GUINT64_FORMAT "B, size %"G_GUINT64_FORMAT "B)",
-			inactive_part, dest_partition.start, dest_partition.size);
+	g_message("Found inactive (%s) half of boot partition region (pos. %"G_GUINT64_FORMAT "B, size %"G_GUINT64_FORMAT "B)",
+			inactive_half == 0 ? "first" : "second", dest_partition.start, dest_partition.size);
 
 	if (dest_partition.size < (guint64)image->checksum.size) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
@@ -1412,7 +1647,24 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Clearing inactive boot partition %d on %s", inactive_part,
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_ACTIVATING"),
+			g_strdup_printf("%d", inactive_half));
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_START"),
+			g_strdup_printf("%"G_GUINT64_FORMAT, dest_partition.start));
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_SIZE"),
+			g_strdup_printf("%"G_GUINT64_FORMAT, dest_partition.size));
+
+	/* run slot pre install hook if enabled */
+	if (hook_name && image->hooks.pre_install) {
+		res = run_slot_hook_extra_env(hook_name, R_SLOT_HOOK_PRE_INSTALL, image,
+				dest_slot, vars, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+	g_message("Clearing inactive (%s) half of boot partition region on %s", inactive_half == 0 ? "first" : "second",
 			dest_slot->device);
 
 	res = clear_boot_switch_partition(dest_slot->device, &dest_partition, &ierror);
@@ -1422,7 +1674,7 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Opening inactive boot partition %d on %s", inactive_part,
+	g_message("Opening inactive (%s) half of boot partition region on %s", inactive_half == 0 ? "first" : "second",
 			dest_slot->device);
 
 	out_fd = open(dest_slot->device, O_WRONLY);
@@ -1451,8 +1703,8 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Copying image to inactive boot partition %d on %s",
-			inactive_part, dest_slot->device);
+	g_message("Copying image to inactive (%s) half of boot partition region on %s",
+			inactive_half == 0 ? "first" : "second", dest_slot->device);
 
 	res = copy_raw_image(image, outstream, &ierror);
 	if (!res) {
@@ -1460,9 +1712,7 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 		goto out;
 	}
 
-	g_message("Setting GPT to switch boot partition");
-
-	res = r_gpt_switch_set_boot_partition(dest_slot->device, &dest_partition, &ierror);
+	res = g_output_stream_close(G_OUTPUT_STREAM(outstream), NULL, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1470,13 +1720,21 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL,
-				dest_slot, &ierror);
+		res = run_slot_hook_extra_env(hook_name, R_SLOT_HOOK_POST_INSTALL, image,
+				dest_slot, vars, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
 		}
 	}
+
+	g_message("Setting %s half of boot partition region active in GPT", inactive_half == 0 ? "first" : "second");
+
+	res = r_gpt_switch_set_boot_partition(dest_slot->device, &dest_partition, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
 out:
 	if (out_fd >= 0)
 		close(out_fd);
@@ -1496,20 +1754,7 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 	g_autoptr(GUnixOutputStream) outstream = NULL;
 	GError *ierror = NULL;
 	g_autoptr(RaucSlot) part_slot = NULL;
-
-	/* run slot pre install hook if enabled */
-	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(
-				hook_name,
-				R_SLOT_HOOK_PRE_INSTALL,
-				NULL,
-				dest_slot,
-				&ierror);
-		if (!res) {
-			g_propagate_error(error, ierror);
-			goto out;
-		}
-	}
+	g_autoptr(GHashTable) vars = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
 	realdev = r_realpath(dest_slot->device);
 	if (!realdev) {
@@ -1557,6 +1802,24 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 		goto out;
 	}
 
+	g_hash_table_insert(vars, g_strdup("RAUC_BOOT_PARTITION_ACTIVATING"),
+			g_strdup_printf("%d", INACTIVE_BOOT_PARTITION(part_active)));
+
+	/* run slot pre install hook if enabled */
+	if (hook_name && image->hooks.pre_install) {
+		res = run_slot_hook_extra_env(
+				hook_name,
+				R_SLOT_HOOK_PRE_INSTALL,
+				image,
+				dest_slot,
+				vars,
+				&ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
 	/* clear block device partition */
 	g_message("Clearing slot device %s", part_slot->device);
 	res = clear_slot(part_slot, &ierror);
@@ -1583,7 +1846,28 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 		goto out;
 	}
 
-	/* reenable read-only on determined eMMC boot partition */
+	res = g_output_stream_close(G_OUTPUT_STREAM(outstream), NULL, &ierror);
+	if (!res) {
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
+	/* run slot post install hook if enabled */
+	if (hook_name && image->hooks.post_install) {
+		res = run_slot_hook_extra_env(
+				hook_name,
+				R_SLOT_HOOK_POST_INSTALL,
+				image,
+				dest_slot,
+				vars,
+				&ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	}
+
+	/* re-enable read-only on determined eMMC boot partition */
 	g_debug("Reenabling read-only mode of slot device partition %s",
 			part_slot->device);
 	res = r_emmc_force_part_ro(part_slot->device, &ierror);
@@ -1628,20 +1912,6 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 
 	g_message("Boot partition %s is now active", part_slot->device);
 
-	/* run slot post install hook if enabled */
-	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(
-				hook_name,
-				R_SLOT_HOOK_POST_INSTALL,
-				NULL,
-				dest_slot,
-				&ierror);
-		if (!res) {
-			g_propagate_error(error, ierror);
-			goto out;
-		}
-	}
-
 out:
 	/* ensure that the eMMC boot partition is read-only afterwards */
 	if (!res && part_slot)
@@ -1658,7 +1928,7 @@ static gboolean img_to_raw_handler(RaucImage *image, RaucSlot *dest_slot, const
 
 	/* run slot pre install hook if enabled */
 	if (hook_name && image->hooks.pre_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_PRE_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1674,7 +1944,7 @@ static gboolean img_to_raw_handler(RaucImage *image, RaucSlot *dest_slot, const
 
 	/* run slot post install hook if enabled */
 	if (hook_name && image->hooks.post_install) {
-		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, NULL, dest_slot, &ierror);
+		res = run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, image, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
@@ -1691,7 +1961,6 @@ static gboolean hook_install_handler(RaucImage *image, RaucSlot *dest_slot, cons
 	gboolean res = FALSE;
 
 	/* run slot install hook */
-	g_message("Running custom slot install hook for %s", dest_slot->name);
 	res = run_slot_hook(hook_name, R_SLOT_HOOK_INSTALL, image, dest_slot, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
@@ -1718,6 +1987,15 @@ RaucUpdatePair updatepairs[] = {
 	{"*.img.caibx", "ubivol", img_to_ubivol_handler},
 	{"*.img.caibx", "ubifs", img_to_ubifs_handler},
 	{"*.squashfs.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-lz4.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-lzo.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-xz.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-zst.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs.caibx", "raw", img_to_raw_handler},
+	{"*.squashfs-lz4.caibx", "raw", img_to_raw_handler},
+	{"*.squashfs-lzo.caibx", "raw", img_to_raw_handler},
+	{"*.squashfs-xz.caibx", "raw", img_to_raw_handler},
+	{"*.squashfs-zst.caibx", "raw", img_to_raw_handler},
 	{"*.caidx", "ext4", archive_to_ext4_handler},
 	{"*.caidx", "ubifs", archive_to_ubifs_handler},
 	{"*.caidx", "vfat", archive_to_vfat_handler},
@@ -1725,19 +2003,29 @@ RaucUpdatePair updatepairs[] = {
 	{"*.ext4", "raw", img_to_raw_handler},
 	{"*.vfat", "raw", img_to_raw_handler},
 	{"*.squashfs", "raw", img_to_raw_handler},
+	{"*.squashfs-lz4", "raw", img_to_raw_handler},
+	{"*.squashfs-lzo", "raw", img_to_raw_handler},
+	{"*.squashfs-xz", "raw", img_to_raw_handler},
+	{"*.squashfs-zst", "raw", img_to_raw_handler},
 	{"*.vfat", "vfat", img_to_fs_handler},
 	{"*.tar*", "ext4", archive_to_ext4_handler},
 	{"*.catar", "ext4", archive_to_ext4_handler},
 	{"*.tar*", "ubifs", archive_to_ubifs_handler},
+	{"*.tar*", "jffs2", archive_to_jffs2_handler},
 	{"*.tar*", "vfat", archive_to_vfat_handler},
 	{"*.ubifs", "ubivol", img_to_ubivol_handler},
 	{"*.ubifs", "ubifs", img_to_ubifs_handler},
 	{"*.img", "ext4", img_to_fs_handler},
+	{"*.img", "nor", img_to_nor_handler},
 	{"*.img", "nand", img_to_nand_handler},
 	{"*.img", "ubifs", img_to_ubifs_handler},
 	{"*.img", "ubivol", img_to_ubivol_handler},
 	{"*.img", "vfat", img_to_fs_handler},
 	{"*.squashfs", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-lz4", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-lzo", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-xz", "ubivol", img_to_ubivol_handler},
+	{"*.squashfs-zst", "ubivol", img_to_ubivol_handler},
 #if ENABLE_EMMC_BOOT_SUPPORT == 1
 	{"*.img", "boot-emmc", img_to_boot_emmc_handler},
 	{"*", "boot-emmc", NULL},
