@@ -19,6 +19,15 @@ static void bootchooser_fixture_set_up(BootchooserFixture *fixture,
 	g_assert_nonnull(fixture->tmpdir);
 }
 
+static void custom_bootchooser_fixture_set_up(BootchooserFixture *fixture,
+		gconstpointer user_data)
+{
+	bootchooser_fixture_set_up(fixture, user_data);
+
+	g_assert_true(test_copy_file("test/bin/custom-bootloader-script", NULL,
+			fixture->tmpdir, "custom-bootloader-script"));
+}
+
 static void bootchooser_fixture_tear_down(BootchooserFixture *fixture,
 		gconstpointer user_data)
 {
@@ -519,9 +528,9 @@ ORDER=B A\n\
  * BOOT_B_LEFT=3\n\
  * "
  */
-static void test_uboot_initialize_state(const gchar *vars)
+static void test_uboot_initialize_state(const BootchooserFixture *fixture, const gchar *vars)
 {
-	g_autofree gchar *state_path = g_build_filename(g_get_tmp_dir(), "uboot-test-state", NULL);
+	g_autofree gchar *state_path = g_build_filename(fixture->tmpdir, "uboot-test-state", NULL);
 	g_setenv("UBOOT_STATE_PATH", state_path, TRUE);
 	g_assert_true(g_file_set_contents(state_path, vars, -1, NULL));
 }
@@ -534,9 +543,9 @@ static void test_uboot_initialize_state(const gchar *vars)
  * Returns TRUE if mock tools state content equals desired content,
  * FALSE otherwise
  */
-static gboolean test_uboot_post_state(const gchar *compare)
+static gboolean test_uboot_post_state(const BootchooserFixture *fixture, const gchar *compare)
 {
-	g_autofree gchar *state_path = g_build_filename(g_get_tmp_dir(), "uboot-test-state", NULL);
+	g_autofree gchar *state_path = g_build_filename(fixture->tmpdir, "uboot-test-state", NULL);
 	g_autofree gchar *contents = NULL;
 
 	g_assert_true(g_file_get_contents(state_path, &contents, NULL, NULL));
@@ -596,7 +605,7 @@ bootname=B\n";
 	g_assert_nonnull(rootfs1);
 
 	/* check rootfs.0 and rootfs.1 are considered bad (as not in BOOT_ORDER / no attempts left) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=B\n\
 BOOT_A_LEFT=3\n\
 BOOT_B_LEFT=0\n\
@@ -607,7 +616,7 @@ BOOT_B_LEFT=0\n\
 	g_assert_false(good);
 
 	/* check rootfs.1 is considered primary (as rootfs.0 has BOOT_A_LEFT set to 0) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=A B\n\
 BOOT_A_LEFT=0\n\
 BOOT_B_LEFT=3\n\
@@ -618,7 +627,7 @@ BOOT_B_LEFT=3\n\
 	g_assert(primary == rootfs1);
 
 	/* check none is considered primary (as rootfs.0 has BOOT_A_LEFT set to 0 and rootfs.1 is not in BOOT_ORDER) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=A\n\
 BOOT_A_LEFT=0\n\
 BOOT_B_LEFT=3\n\
@@ -629,7 +638,7 @@ BOOT_B_LEFT=3\n\
 	g_clear_error(&error);
 
 	/* check rootfs.0 + rootfs.1 are considered good */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=A B\n\
 BOOT_A_LEFT=3\n\
 BOOT_B_LEFT=3\n\
@@ -641,7 +650,7 @@ BOOT_B_LEFT=3\n\
 
 	/* check rootfs.0 is marked bad (BOOT_A_LEFT set to 0) */
 	g_assert_true(r_boot_set_state(rootfs0, FALSE, NULL));
-	g_assert_true(test_uboot_post_state("\
+	g_assert_true(test_uboot_post_state(fixture, "\
 BOOT_ORDER=B\n\
 BOOT_A_LEFT=0\n\
 BOOT_B_LEFT=3\n\
@@ -652,33 +661,33 @@ BOOT_B_LEFT=3\n\
 
 	/* check rootfs.0 is marked good again (BOOT_A_LEFT reset to 3) */
 	g_assert_true(r_boot_set_state(rootfs0, TRUE, NULL));
-	g_assert_true(test_uboot_post_state("\
+	g_assert_true(test_uboot_post_state(fixture, "\
 BOOT_ORDER=B\n\
 BOOT_A_LEFT=3\n\
 BOOT_B_LEFT=3\n\
 "));
 
 	/* check rootfs.1 is marked primary (first in BOOT_ORDER, BOOT_B_LEFT reset to 3) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=A B\n\
 BOOT_A_LEFT=3\n\
 BOOT_B_LEFT=1\n\
 ");
 	g_assert_true(r_boot_set_primary(rootfs1, NULL));
-	g_assert_true(test_uboot_post_state("\
+	g_assert_true(test_uboot_post_state(fixture, "\
 BOOT_ORDER=B A\n\
 BOOT_A_LEFT=3\n\
 BOOT_B_LEFT=3\n\
 "));
 
 	/* check rootfs.1 is marked primary while rootfs.0 remains disabled (BOOT_A_LEFT remains 0)  */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=A B\n\
 BOOT_A_LEFT=0\n\
 BOOT_B_LEFT=0\n\
 ");
 	g_assert_true(r_boot_set_primary(rootfs1, NULL));
-	g_assert_true(test_uboot_post_state("\
+	g_assert_true(test_uboot_post_state(fixture, "\
 BOOT_ORDER=B A\n\
 BOOT_A_LEFT=0\n\
 BOOT_B_LEFT=3\n\
@@ -727,7 +736,7 @@ bootname=A\n";
 	g_assert_nonnull(rootfs0);
 
 	/* check rootfs.0 is marked bad (not in BOOT_ORDER, BOOT_R_LEFT = 0) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=R A\n\
 BOOT_R_LEFT=3\n\
 BOOT_A_LEFT=3\n\
@@ -735,14 +744,14 @@ BOOT_A_LEFT=3\n\
 	res = r_boot_set_state(rootfs0, FALSE, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
-	g_assert_true(test_uboot_post_state("\
+	g_assert_true(test_uboot_post_state(fixture, "\
 BOOT_ORDER=R\n\
 BOOT_R_LEFT=3\n\
 BOOT_A_LEFT=0\n\
 "));
 
 	/* check rootfs.0 is marked primary (first in BOOT_ORDER) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=R A\n\
 BOOT_R_LEFT=3\n\
 BOOT_A_LEFT=1\n\
@@ -750,14 +759,14 @@ BOOT_A_LEFT=1\n\
 	res = r_boot_set_primary(rootfs0, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
-	g_assert_true(test_uboot_post_state("\
+	g_assert_true(test_uboot_post_state(fixture, "\
 BOOT_ORDER=A R\n\
 BOOT_R_LEFT=3\n\
 BOOT_A_LEFT=3\n\
 "));
 
 	/* check rootfs.0 is considered primary (as rootfs.0 has BOOT_A_LEFT set to 0) */
-	test_uboot_initialize_state("\
+	test_uboot_initialize_state(fixture, "\
 BOOT_ORDER=A R\n\
 BOOT_R_LEFT=3\n\
 BOOT_A_LEFT=3\n\
@@ -824,6 +833,220 @@ bootname=system1\n";
 	g_assert_true(r_boot_set_primary(slot, NULL));
 }
 
+/* Write content to state storage for custom-backend RAUC mock
+ * tools. Content should be similar to:
+ * "\
+ * PRIMARY=A\n\
+ * STATE_A=good\n\
+ * STATE_B=good\n\
+ * "
+ */
+static void test_custom_initialize_state(const BootchooserFixture *fixture, const gchar *vars)
+{
+	g_autofree gchar *state_path = g_build_filename(fixture->tmpdir, "custom-test-state", NULL);
+	g_setenv("CUSTOM_STATE_PATH", state_path, TRUE);
+	g_assert_true(g_file_set_contents(state_path, vars, -1, NULL));
+}
+
+/* Content written should identical to format described for
+ * test_custom_backend_initialize_state().
+ *
+ * Returns TRUE if mock tools state content equals desired content,
+ * FALSE otherwise
+ */
+static gboolean test_custom_post_state(const BootchooserFixture *fixture, const gchar *compare)
+{
+	g_autofree gchar *state_path = g_build_filename(fixture->tmpdir, "custom-test-state", NULL);
+	g_autofree gchar *contents = NULL;
+
+	g_assert_true(g_file_get_contents(state_path, &contents, NULL, NULL));
+
+	if (g_strcmp0(contents, compare) != 0) {
+		g_print("Error: '%s' and '%s' differ\n", contents, compare);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static void bootchooser_custom(BootchooserFixture *fixture,
+		gconstpointer user_data)
+{
+	RaucSlot *rootfs0 = NULL;
+	RaucSlot *rootfs1 = NULL;
+	RaucSlot *primary = NULL;
+	gboolean good;
+	GError *error = NULL;
+	gboolean res;
+
+	const gchar *cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=custom\n\
+mountprefix=/mnt/myrauc/\n\
+\n\
+[handlers]\n\
+bootloader-custom-backend=custom-bootloader-script\n\
+\n\
+[keyring]\n\
+path=/etc/rauc/keyring/\n\
+\n\
+[slot.rescue.0]\n\
+device=/dev/mtd4\n\
+type=raw\n\
+bootname=R\n\
+readonly=true\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/rootfs-0\n\
+type=ext4\n\
+bootname=A\n\
+\n\
+[slot.rootfs.1]\n\
+device=/dev/rootfs-1\n\
+type=ext4\n\
+bootname=B\n";
+
+	gchar* pathname = write_tmp_file(fixture->tmpdir, "custom.conf", cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	g_clear_pointer(&r_context_conf()->configpath, g_free);
+	r_context_conf()->configpath = pathname;
+	r_context();
+
+	rootfs0 = find_config_slot_by_device(r_context()->config, "/dev/rootfs-0");
+	g_assert_nonnull(rootfs0);
+	rootfs1 = find_config_slot_by_device(r_context()->config, "/dev/rootfs-1");
+	g_assert_nonnull(rootfs1);
+
+	/* check A and B can be set to bad */
+	test_custom_initialize_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=good\n\
+STATE_B=good\n\
+");
+	res = r_boot_get_state(rootfs0, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_true(good);
+	res = r_boot_get_state(rootfs1, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_true(good);
+
+	res = r_boot_set_state(rootfs0, FALSE, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	res = r_boot_get_state(rootfs0, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_false(good);
+	res = r_boot_set_state(rootfs1, FALSE, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	res = r_boot_get_state(rootfs1, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_false(good);
+	g_assert_true(test_custom_post_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=bad\n\
+STATE_B=bad\n\
+"));
+
+	/* check A and B can be set to good */
+	test_custom_initialize_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=bad\n\
+STATE_B=bad\n\
+");
+	res = r_boot_get_state(rootfs0, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_false(good);
+	res = r_boot_get_state(rootfs1, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_false(good);
+	res = r_boot_set_state(rootfs0, TRUE, &error);
+	res = r_boot_get_state(rootfs0, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_true(good);
+	res = r_boot_set_state(rootfs1, TRUE, &error);
+	res = r_boot_get_state(rootfs1, &good, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	g_assert_true(good);
+	g_assert_true(test_custom_post_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=good\n\
+STATE_B=good\n\
+"));
+
+	/* check B can be set to primary */
+	test_custom_initialize_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=good\n\
+STATE_B=good\n\
+");
+	primary = r_boot_get_primary(&error);
+	g_assert_no_error(error);
+	g_assert_nonnull(primary);
+	g_assert(primary == rootfs0);
+	g_assert(primary != rootfs1);
+
+	res = r_boot_set_primary(rootfs1, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	primary = r_boot_get_primary(&error);
+	g_assert_no_error(error);
+	g_assert_nonnull(primary);
+	g_assert(primary != rootfs0);
+	g_assert(primary == rootfs1);
+	g_assert_true(test_custom_post_state(fixture, "\
+PRIMARY=B\n\
+STATE_A=good\n\
+STATE_B=good\n\
+"));
+
+	/* check A can be set to primary */
+	test_custom_initialize_state(fixture, "\
+PRIMARY=B\n\
+STATE_A=good\n\
+STATE_B=good\n\
+");
+	primary = r_boot_get_primary(&error);
+	g_assert_no_error(error);
+	g_assert_nonnull(primary);
+	g_assert(primary != rootfs0);
+	g_assert(primary == rootfs1);
+
+	res = r_boot_set_primary(rootfs0, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+	primary = r_boot_get_primary(&error);
+	g_assert_no_error(error);
+	g_assert_nonnull(primary);
+	g_assert(primary == rootfs0);
+	g_assert(primary != rootfs1);
+	g_assert_true(test_custom_post_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=good\n\
+STATE_B=good\n\
+"));
+
+	/* check none is considered primary if both are bad */
+	test_custom_initialize_state(fixture, "\
+PRIMARY=A\n\
+STATE_A=bad\n\
+STATE_B=bad\n\
+");
+	primary = r_boot_get_primary(&error);
+	g_assert_null(primary);
+	g_assert_error(error, R_BOOTCHOOSER_ERROR, R_BOOTCHOOSER_ERROR_PARSE_FAILED);
+	g_clear_error(&error);
+}
 
 int main(int argc, char *argv[])
 {
@@ -860,5 +1083,9 @@ int main(int argc, char *argv[])
 			bootchooser_fixture_set_up, bootchooser_efi,
 			bootchooser_fixture_tear_down);
 
+	g_test_add("/bootchooser/custom", BootchooserFixture, NULL,
+			custom_bootchooser_fixture_set_up, bootchooser_custom,
+			bootchooser_fixture_tear_down);
+
 	return g_test_run();
 }
