@@ -130,7 +130,7 @@ static void test_check_empty_bundle(BundleFixture *fixture,
 	bundlename = write_random_file(fixture->tmpdir, "bundle.raucb", 0, 1234);
 	g_assert_nonnull(bundlename);
 
-	res = check_bundle(bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_false(res);
 	g_assert_error(ierror, G_IO_ERROR, G_IO_ERROR_PARTIAL_INPUT);
 	g_assert_null(bundle);
@@ -147,7 +147,7 @@ static void test_check_invalid_bundle(BundleFixture *fixture,
 	bundlename = write_random_file(fixture->tmpdir, "bundle.raucb", 1024, 1234);
 	g_assert_nonnull(bundlename);
 
-	res = check_bundle(bundlename, &bundle, FALSE, &ierror);
+	res = check_bundle(bundlename, &bundle, CHECK_BUNDLE_NO_VERIFY, &ierror);
 	g_assert_false(res);
 	g_assert_error(ierror, R_BUNDLE_ERROR, R_BUNDLE_ERROR_IDENTIFIER);
 	g_assert_null(bundle);
@@ -160,7 +160,7 @@ static void bundle_test_create_check_error(BundleFixture *fixture,
 	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_false(res);
 	g_assert_error(ierror, R_BUNDLE_ERROR, R_BUNDLE_ERROR_FORMAT);
 	g_assert_null(bundle);
@@ -178,7 +178,7 @@ static void bundle_test_create_extract(BundleFixture *fixture,
 	outputdir = g_build_filename(fixture->tmpdir, "output", NULL);
 	g_assert_nonnull(outputdir);
 
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -211,7 +211,7 @@ static void bundle_test_create_mount_extract(BundleFixture *fixture,
 	if (!test_running_as_root())
 		return;
 
-	res = check_bundle(fixture->bundlename, &bundle, FALSE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_NO_VERIFY, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -225,41 +225,30 @@ static void bundle_test_create_mount_extract(BundleFixture *fixture,
 	g_assert_true(res);
 }
 
-static void bundle_test_extract_manifest(BundleFixture *fixture,
+static void bundle_test_extract_signature(BundleFixture *fixture,
 		gconstpointer user_data)
 {
-	g_autofree gchar *outputdir = NULL;
-	g_autofree gchar *filepath = NULL;
+	g_autofree gchar *outputsig = NULL;
 	g_autoptr(RaucBundle) bundle = NULL;
 	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
 
-	outputdir = g_build_filename(fixture->tmpdir, "output", NULL);
-	g_assert_nonnull(outputdir);
+	outputsig = g_build_filename(fixture->tmpdir, "bundle.sig", NULL);
+	g_assert_nonnull(outputsig);
 
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
 
-	res = extract_file_from_bundle(bundle, outputdir, "manifest.raucm", &ierror);
+	res = extract_signature(bundle, outputsig, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 
-	filepath = g_build_filename(outputdir, "manifest.raucm", NULL);
-	g_assert_true(g_file_test(filepath, G_FILE_TEST_IS_REGULAR));
-	g_clear_pointer(&filepath, g_free);
-
-	filepath = g_build_filename(outputdir, "rootfs.ext4", NULL);
-	g_assert_false(g_file_test(filepath, G_FILE_TEST_EXISTS));
-	g_clear_pointer(&filepath, g_free);
-
-	filepath = g_build_filename(outputdir, "appfs.ext4", NULL);
-	g_assert_false(g_file_test(filepath, G_FILE_TEST_EXISTS));
-	g_clear_pointer(&filepath, g_free);
+	g_assert_true(g_file_test(outputsig, G_FILE_TEST_IS_REGULAR));
+	g_clear_pointer(&outputsig, g_free);
 }
 
-
 static void assert_casync_manifest(RaucManifest *rm)
 {
 	RaucImage *test_img = NULL;
@@ -296,7 +285,7 @@ static void bundle_test_check_casync_old(BundleFixture *fixture, gconstpointer u
 	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
 			"Verifying bundle signature*" );
 
-	res = check_bundle(bundlepath, &bundle, TRUE, &ierror);
+	res = check_bundle(bundlepath, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -325,7 +314,7 @@ static void bundle_test_check_casync_new(BundleFixture *fixture, gconstpointer u
 	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
 			"Verifying bundle signature*" );
 
-	res = check_bundle(bundlepath, &bundle, TRUE, &ierror);
+	res = check_bundle(bundlepath, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -341,6 +330,123 @@ static void bundle_test_check_casync_new(BundleFixture *fixture, gconstpointer u
 // Hack to pull-in context for testing modification
 extern RaucContext *context;
 
+static void bundle_test_replace_signature(BundleFixture *fixture,
+		gconstpointer user_data)
+{
+	g_autofree gchar *resignbundle = NULL;
+	g_autofree gchar *replacebundle = NULL;
+	g_autofree gchar *sigpath = NULL;
+	g_autoptr(RaucBundle) bundle = NULL;
+	g_autoptr(GError) ierror = NULL;
+	gboolean res = FALSE;
+
+	replacebundle = g_build_filename(fixture->tmpdir, "replaced-bundle.raucb", NULL);
+	g_assert_nonnull(replacebundle);
+	resignbundle = g_build_filename(fixture->tmpdir, "resigned-bundle.raucb", NULL);
+	g_assert_nonnull(resignbundle);
+
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/rel-ca.pem");
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
+	g_clear_error(&ierror);
+	g_assert_false(res);
+	g_clear_pointer(&bundle, free_bundle);
+
+	/* Verify input bundle with 'dev' keyring */
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/dev-only-ca.pem");
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+
+	/* Use 'rel' key pair for resigning */
+	context->certpath = g_strdup("test/openssl-ca/rel/release-1.cert.pem");
+	context->keypath = g_strdup("test/openssl-ca/rel/private/release-1.pem");
+	context->signing_keyringpath = g_strdup("test/openssl-ca/rel-ca.pem");
+
+	/* Resign bundle with 'rel' key to extract the signature below */
+	res = resign_bundle(bundle, resignbundle, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_clear_pointer(&bundle, free_bundle);
+
+	/* Verify resigned bundle with old 'dev' key */
+	res = check_bundle(resignbundle, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
+	g_clear_error(&ierror);
+	g_assert_false(res);
+
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/rel-ca.pem");
+	res = check_bundle(resignbundle, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+
+	sigpath = g_build_filename(fixture->tmpdir, "bundle.sig", NULL);
+	g_assert_nonnull(sigpath);
+
+	/* Extract 'rel' signature to replace it in the bundle */
+	res = extract_signature(bundle, sigpath, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_true(g_file_test(sigpath, G_FILE_TEST_IS_REGULAR));
+	g_clear_pointer(&bundle, free_bundle);
+
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/dev-only-ca.pem");
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+
+	res = replace_signature(bundle, sigpath, replacebundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_clear_pointer(&bundle, free_bundle);
+
+	res = check_bundle(replacebundle, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
+	g_clear_error(&ierror);
+	g_assert_false(res);
+	g_clear_pointer(&bundle, free_bundle);
+
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/rel-ca.pem");
+	res = check_bundle(replacebundle, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_clear_pointer(&bundle, free_bundle);
+
+	/* Test without verify */
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/dev-only-ca.pem");
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+
+	/* Test will fail as we trying to replace existing bundle */
+	res = replace_signature(bundle, sigpath, replacebundle, CHECK_BUNDLE_NO_VERIFY, &ierror);
+	g_assert_error(ierror, G_FILE_ERROR, G_FILE_ERROR_EXIST);
+	g_clear_error(&ierror);
+	g_assert_false(res);
+
+	/* Now it should works */
+	g_remove(replacebundle);
+	res = replace_signature(bundle, sigpath, replacebundle, CHECK_BUNDLE_NO_VERIFY, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_clear_pointer(&bundle, free_bundle);
+	g_clear_pointer(&sigpath, g_free);
+
+	res = check_bundle(replacebundle, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
+	g_clear_pointer(&bundle, free_bundle);
+	g_clear_error(&ierror);
+	g_assert_false(res);
+
+	r_context()->config->keyring_path = g_strdup("test/openssl-ca/rel-ca.pem");
+	res = check_bundle(replacebundle, &bundle, CHECK_BUNDLE_TRUST_ENV, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+
+	// hacky restore of original signing_keyringpath
+	context->signing_keyringpath = NULL;
+}
+
 static void bundle_test_resign(BundleFixture *fixture,
 		gconstpointer user_data)
 {
@@ -357,7 +463,7 @@ static void bundle_test_resign(BundleFixture *fixture,
 	 * the context's 'pending' flag which would cause a re-initialization
 	 * of context and thus overwrite content of 'config' member. */
 	r_context()->config->keyring_path = g_strdup("test/openssl-ca/rel-ca.pem");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_clear_error(&ierror);
 	g_assert_false(res);
@@ -366,7 +472,7 @@ static void bundle_test_resign(BundleFixture *fixture,
 
 	/* Verify input bundle with 'dev' keyring */
 	r_context()->config->keyring_path = g_strdup("test/openssl-ca/dev-only-ca.pem");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 
@@ -387,7 +493,7 @@ static void bundle_test_resign(BundleFixture *fixture,
 	 * installing development bundles as well as moving to production
 	 * bundles. */
 	r_context()->config->keyring_path = g_strdup("test/openssl-ca/dev-only-ca.pem");
-	res = check_bundle(resignbundle, &bundle, TRUE, &ierror);
+	res = check_bundle(resignbundle, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_clear_error(&ierror);
 	g_assert_false(res);
@@ -396,7 +502,7 @@ static void bundle_test_resign(BundleFixture *fixture,
 
 	/* Verify resigned bundle with rel keyring */
 	r_context()->config->keyring_path = g_strdup("test/openssl-ca/rel-ca.pem");
-	res = check_bundle(resignbundle, &bundle, TRUE, &ierror);
+	res = check_bundle(resignbundle, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 
@@ -411,7 +517,7 @@ static void bundle_test_wrong_capath(BundleFixture *fixture,
 	g_autoptr(GError) ierror = NULL;
 	r_context()->config->keyring_path = g_strdup("does/not/exist.pem");
 
-	g_assert_false(check_bundle(fixture->bundlename, &bundle, TRUE, &ierror));
+	g_assert_false(check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror));
 	g_assert_null(bundle);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_CA_LOAD);
 
@@ -434,7 +540,7 @@ static void bundle_test_verify_no_crl_warn(BundleFixture *fixture,
 			"Reading bundle*");
 	g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
 			"Detected CRL but CRL checking is disabled!");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -448,7 +554,7 @@ static void bundle_test_verify_revoked(BundleFixture *fixture,
 	g_autoptr(RaucBundle) bundle = NULL;
 	g_autoptr(GError) ierror = NULL;
 
-	g_assert_false(check_bundle(fixture->bundlename, &bundle, TRUE, &ierror));
+	g_assert_false(check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror));
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_assert_null(bundle);
 }
@@ -464,7 +570,7 @@ static void bundle_test_purpose_default(BundleFixture *fixture,
 
 	g_message("testing default purpose with default cert");
 	r_context()->config->keyring_check_purpose = NULL;
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -472,7 +578,7 @@ static void bundle_test_purpose_default(BundleFixture *fixture,
 
 	g_message("testing purpose 'smimesign' with default cert");
 	r_context()->config->keyring_check_purpose = g_strdup("smimesign");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -480,14 +586,14 @@ static void bundle_test_purpose_default(BundleFixture *fixture,
 
 	g_message("testing purpose 'codesign' with default cert");
 	r_context()->config->keyring_check_purpose = g_strdup("codesign");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_clear_error(&ierror);
 	g_assert_false(res);
 
 	g_message("testing purpose 'any' with default cert");
 	r_context()->config->keyring_check_purpose = g_strdup("any");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -507,7 +613,7 @@ static void bundle_test_purpose_email(BundleFixture *fixture,
 
 	g_message("testing default purpose with 'smimesign' cert");
 	r_context()->config->keyring_check_purpose = NULL;
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -515,7 +621,7 @@ static void bundle_test_purpose_email(BundleFixture *fixture,
 
 	g_message("testing purpose 'smimesign' with 'smimesign' cert");
 	r_context()->config->keyring_check_purpose = g_strdup("smimesign");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -523,7 +629,7 @@ static void bundle_test_purpose_email(BundleFixture *fixture,
 
 	g_message("testing purpose 'codesign' with 'smimesign' cert");
 	r_context()->config->keyring_check_purpose = g_strdup("codesign");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_clear_error(&ierror);
 	g_assert_false(res);
@@ -531,7 +637,7 @@ static void bundle_test_purpose_email(BundleFixture *fixture,
 
 	g_message("testing purpose 'any' with 'smimesign' cert");
 	r_context()->config->keyring_check_purpose = g_strdup("any");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -551,7 +657,7 @@ static void bundle_test_purpose_codesign(BundleFixture *fixture,
 
 	g_message("testing default purpose with 'codesign' cert");
 	r_context()->config->keyring_check_purpose = NULL;
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_clear_error(&ierror);
 	g_assert_false(res);
@@ -559,7 +665,7 @@ static void bundle_test_purpose_codesign(BundleFixture *fixture,
 
 	g_message("testing purpose 'smimesign' with 'codesign' cert");
 	r_context()->config->keyring_check_purpose = g_strdup("smimesign");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_error(ierror, R_SIGNATURE_ERROR, R_SIGNATURE_ERROR_INVALID);
 	g_clear_error(&ierror);
 	g_assert_false(res);
@@ -567,7 +673,7 @@ static void bundle_test_purpose_codesign(BundleFixture *fixture,
 
 	g_message("testing purpose 'codesign' with 'codesign' cert");
 	r_context()->config->keyring_check_purpose = g_strdup("codesign");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -575,7 +681,7 @@ static void bundle_test_purpose_codesign(BundleFixture *fixture,
 
 	g_message("testing purpose 'any' with 'codesign' cert");
 	r_context()->config->keyring_check_purpose = g_strdup("any");
-	res = check_bundle(fixture->bundlename, &bundle, TRUE, &ierror);
+	res = check_bundle(fixture->bundlename, &bundle, CHECK_BUNDLE_DEFAULT, &ierror);
 	g_assert_no_error(ierror);
 	g_assert_true(res);
 	g_assert_nonnull(bundle);
@@ -623,9 +729,9 @@ int main(int argc, char *argv[])
 				bundle_fixture_set_up_bundle, bundle_test_create_mount_extract,
 				bundle_fixture_tear_down);
 
-		g_test_add(g_strdup_printf("/bundle/extract_manifest/%s", format_name),
+		g_test_add(g_strdup_printf("/bundle/extract_signature/%s", format_name),
 				BundleFixture, bundle_data,
-				bundle_fixture_set_up_bundle, bundle_test_extract_manifest,
+				bundle_fixture_set_up_bundle, bundle_test_extract_signature,
 				bundle_fixture_tear_down);
 
 		g_test_add(g_strdup_printf("/bundle/resign/%s", format_name),
@@ -633,6 +739,11 @@ int main(int argc, char *argv[])
 				bundle_fixture_set_up_bundle, bundle_test_resign,
 				bundle_fixture_tear_down);
 
+		g_test_add(g_strdup_printf("/bundle/replace_signature/%s", format_name),
+				BundleFixture, bundle_data,
+				bundle_fixture_set_up_bundle, bundle_test_replace_signature,
+				bundle_fixture_tear_down);
+
 		g_test_add(g_strdup_printf("/bundle/wrong_capath/%s", format_name),
 				BundleFixture, bundle_data,
 				bundle_fixture_set_up_bundle, bundle_test_wrong_capath,
