@@ -15,28 +15,28 @@ typedef struct {
 	gchar *tmpdir;
 } InstallFixture;
 
-gchar* random_bytes(gsize size, guint32 seed)
+guint8* random_bytes(gsize size, guint32 seed)
 {
-	gchar *str = g_new0(gchar, size + 1);
+	guint8 *bytes = g_malloc0(size);
 	GRand *rand = g_rand_new_with_seed(seed);
 	for (gsize i = 0; i < size; i++) {
-		str[i] = (gchar) g_rand_int(rand) & 0xFF;
+		bytes[i] = g_rand_int(rand) & 0xFF;
 	}
-	return str;
+	return bytes;
 }
 
 gchar* write_random_file(const gchar *tmpdir, const gchar *filename,
 		gsize size, const guint32 seed)
 {
 	gchar *pathname;
-	gchar *content;
+	guint8 *content;
 
 	pathname = g_build_filename(tmpdir, filename, NULL);
 	g_assert_nonnull(pathname);
 
 	content = random_bytes(size, seed);
 
-	if (!g_file_set_contents(pathname, content, size, NULL)) {
+	if (!g_file_set_contents(pathname, (gchar *)content, size, NULL)) {
 		return NULL;
 	}
 
@@ -168,7 +168,7 @@ gboolean test_rm_tree(const gchar *dirname, const gchar *filename)
 	return res;
 }
 
-int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, gboolean custom_handler, gboolean hooks)
+int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, const ManifestTestOptions *options)
 {
 	gchar *path = g_build_filename(dirname, filename, NULL);
 	RaucManifest *rm = g_new0(RaucManifest, 1);
@@ -177,10 +177,12 @@ int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, gboo
 	rm->update_compatible = g_strdup("Test Config");
 	rm->update_version = g_strdup("2011.03-2");
 
-	if (custom_handler)
+	rm->bundle_format = options->format;
+
+	if (options->custom_handler)
 		rm->handler_name = g_strdup("custom_handler.sh");
 
-	if (hooks) {
+	if (options->hooks) {
 		rm->hook_name = g_strdup("hook.sh");
 	}
 
@@ -188,7 +190,7 @@ int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, gboo
 
 	img->slotclass = g_strdup("rootfs");
 	img->filename = g_strdup("rootfs.ext4");
-	if (hooks)
+	if (options->hooks)
 		img->hooks.post_install = TRUE;
 	rm->images = g_list_append(rm->images, img);
 
@@ -217,6 +219,7 @@ gboolean test_make_filesystem(const gchar *dirname, const gchar *filename)
 			&error,
 			"/sbin/mkfs.ext4",
 			"-F",
+			"-I256",
 			path,
 			NULL);
 
@@ -237,7 +240,7 @@ gboolean test_make_filesystem(const gchar *dirname, const gchar *filename)
 
 gboolean test_mount(const gchar *src, const gchar *dest)
 {
-	return r_mount_full(src, dest, NULL, 0, NULL, NULL);
+	return r_mount_full(src, dest, NULL, NULL, NULL);
 }
 
 
@@ -315,21 +318,22 @@ gboolean test_make_slot_user_writable(const gchar* path, const gchar* file)
 
 	test_do_chmod(mountpath);
 
-	r_umount(mountpath, NULL);
+	g_assert_true(r_umount(mountpath, NULL));
 
 	res = TRUE;
 
 	return res;
 }
 
-void test_create_content(gchar *contentdir)
+void test_create_content(gchar *contentdir, const ManifestTestOptions *options)
 {
 	g_assert(g_mkdir(contentdir, 0777) == 0);
 	g_assert(test_prepare_dummy_file(contentdir, "rootfs.ext4",
 			1024*1024, "/dev/urandom") == 0);
 	g_assert(test_prepare_dummy_file(contentdir, "appfs.ext4",
 			64*1024, "/dev/urandom") == 0);
-	g_assert(test_prepare_manifest_file(contentdir, "manifest.raucm", FALSE, FALSE) == 0);
+	g_assert(test_prepare_manifest_file(contentdir, "manifest.raucm",
+			options) == 0);
 }
 
 void test_create_bundle(gchar *contentdir, gchar *bundlename)
@@ -337,10 +341,6 @@ void test_create_bundle(gchar *contentdir, gchar *bundlename)
 	GError *error = NULL;
 	gboolean res = FALSE;
 
-	res = update_manifest(contentdir, FALSE, &error);
-	g_assert_no_error(error);
-	g_assert_true(res);
-
 	res = create_bundle(bundlename, contentdir, &error);
 	g_assert_no_error(error);
 	g_assert_true(res);
