@@ -106,6 +106,8 @@ filename=appfs.ext4";
 	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, &data->manifest_test_options);
 }
 
+/* Note: Also ensures having no image in a slot with an 'install' per-slot hook
+ * is valid. */
 static void install_fixture_set_up_bundle_install_hook(InstallFixture *fixture,
 		gconstpointer user_data)
 {
@@ -122,7 +124,6 @@ filename=rootfs.ext4\n\
 hooks=install\n\
 \n\
 [image.appfs]\n\
-filename=appfs.ext4\n\
 hooks=install";
 
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
@@ -272,14 +273,6 @@ filename=appfs.ext4\n\
 sha256=ecf4c031d01cb9bfa9aa5ecfce93efcf9149544bdbf91178d2c2d9d1d24076ca\n\
 filename=demofs.ext4\n\
 \n\
-[file.rootfs/vmlinuz]\n\
-sha256=5fb50868cd1f2e34ff531d6680c9b734ba35ed4944072f396a50871e9c2d5155\n\
-filename=linux.img\n\
-\n\
-[file.rootfs/initramfs]\n\
-sha256=d37328d0d80779573b204762ee8aa011c22a5c43088f7541a8c1f591f8e3be6a\n\
-filename=initramfs.cpio.gz\n\
-\n\
 [image.bootloader]\n\
 sha256=ecf4c031d01cb9bfa9aa5ecfce93efcf9149544bdbf91178d2c2d9d1d24076ca\n\
 filename=bootloader.img";
@@ -348,6 +341,8 @@ static void test_install_determine_target_group_non_redundant(void)
 	g_autofree gchar *tmpdir = NULL;
 	g_autofree gchar *sysconfpath = NULL;
 	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
+	gboolean res;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -369,7 +364,9 @@ device=/dev/null\n\
 	r_context_conf()->bootslot = g_strdup("system0");
 	r_context();
 
-	g_assert_true(determine_slot_states(NULL));
+	res = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -379,12 +376,14 @@ device=/dev/null\n\
 }
 
 /* Test a typical asynchronous slot setup (rootfs + rescuefs) with additional
- * childs */
+ * children */
 static void test_install_target_group_async(void)
 {
 	g_autofree gchar *tmpdir = NULL;
 	g_autofree gchar *sysconfpath = NULL;
 	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
+	gboolean res;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -418,7 +417,9 @@ device=/dev/null\n\
 	r_context_conf()->bootslot = g_strdup("rescue");
 	r_context();
 
-	g_assert_true(determine_slot_states(NULL));
+	res = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -432,12 +433,14 @@ device=/dev/null\n\
 	g_assert_cmpstr(((RaucSlot*)g_hash_table_lookup(tgrp, "appfs"))->name, ==, "appfs.0");
 }
 
-/* Test a typical synchronous slot setup (rootfs a + b) with appfs childs */
+/* Test a typical synchronous slot setup (rootfs a + b) with appfs children */
 static void test_install_target_group_sync(void)
 {
 	g_autofree gchar *tmpdir = NULL;
 	g_autofree gchar *sysconfpath = NULL;
 	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
+	gboolean res;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -470,7 +473,9 @@ device=/dev/null\n\
 	r_context_conf()->bootslot = g_strdup("system1");
 	r_context();
 
-	g_assert_true(determine_slot_states(NULL));
+	res = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -490,6 +495,8 @@ static void test_install_target_group_loose(void)
 	g_autofree gchar *tmpdir = NULL;
 	g_autofree gchar *sysconfpath = NULL;
 	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
+	gboolean res;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -517,7 +524,9 @@ device=/dev/null\n\
 	r_context_conf()->bootslot = g_strdup("system0");
 	r_context();
 
-	g_assert_true(determine_slot_states(NULL));
+	res = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -537,6 +546,8 @@ static void test_install_target_group_n_redundant(void)
 	g_autofree gchar *tmpdir = NULL;
 	g_autofree gchar *sysconfpath = NULL;
 	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
+	gboolean res;
 
 	const gchar *system_conf = "\
 [system]\n\
@@ -565,7 +576,9 @@ device=/dev/null\n\
 	r_context_conf()->bootslot = g_strdup("system1");
 	r_context();
 
-	g_assert_true(determine_slot_states(NULL));
+	res = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(res);
 
 	tgrp = determine_target_install_group();
 	g_assert_nonnull(tgrp);
@@ -649,7 +662,7 @@ device=/dev/null\n\
 	g_assert_no_error(error);
 	g_assert_nonnull(selected_images);
 
-	/* We expecte the image selection to return both appfs.img and
+	/* We expect the image selection to return both appfs.img and
 	 * rootfs.img as we have matching slots for them. */
 	g_assert_cmpint(g_list_length(selected_images), ==, 2);
 
@@ -726,6 +739,79 @@ device=/dev/null\n\
 	g_assert_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_FAILED);
 }
 
+/* Test that get_install_images() returns non-NULL if there is no booted slot
+ * but the boot was marked as 'external' explicitly */
+static void test_install_image_selection_boot_external(void)
+{
+	g_autofree gchar *tmpdir = NULL;
+	g_autofree gchar *sysconfpath = NULL;
+	g_autoptr(GBytes) data = NULL;
+	g_autoptr(RaucManifest) rm = NULL;
+	g_autoptr(GHashTable) tgrp = NULL;
+	g_autoptr(GError) error = NULL;
+	GList *selected_images = NULL;
+	RaucImage *image = NULL;
+	gboolean res;
+
+#define MANIFEST3 "\
+[update]\n\
+compatible=foo\n\
+\n\
+[image.rootfs]\n\
+filename=rootfs.img\n\
+"
+
+	const gchar *system_conf = "\
+[system]\n\
+compatible=foo\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+bootname=system0\n\
+device=/dev/null\n\
+\n\
+[slot.rootfs.1]\n\
+bootname=system1\n\
+device=/dev/null\n\
+";
+	tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
+
+	sysconfpath = write_tmp_file(tmpdir, "test.conf", system_conf, NULL);
+	g_assert_nonnull(sysconfpath);
+
+	/* Set up context */
+	r_context_conf()->configpath = sysconfpath;
+	r_context_conf()->bootslot = g_strdup("_external_"); /* mark as external */
+	r_context();
+
+	g_message("Bootname is: %s", r_context()->bootslot);
+
+	data = g_bytes_new_static(MANIFEST3, sizeof(MANIFEST3));
+	res = load_manifest_mem(data, &rm, &error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+
+	res = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(res);
+
+	tgrp = determine_target_install_group();
+	g_assert_nonnull(tgrp);
+
+	/* we expect the image mapping to fail as there is no slot candidate
+	 * for image.appfs */
+	selected_images = get_install_images(rm, tgrp, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(selected_images);
+
+	/* We expect a single rootfs slot to match */
+	g_assert_cmpint(g_list_length(selected_images), ==, 1);
+
+	image = (RaucImage*) g_list_nth_data(selected_images, 0);
+	g_assert_nonnull(image);
+	g_assert_cmpstr(image->filename, ==, "rootfs.img");
+}
+
 static void test_install_image_readonly(void)
 {
 	g_autofree gchar *tmpdir = NULL;
@@ -1245,6 +1331,8 @@ int main(int argc, char *argv[])
 
 	g_test_add_func("/install/image-selection/non-matching", test_install_image_selection_no_matching_slot);
 
+	g_test_add_func("/install/image-selection/boot-external", test_install_image_selection_boot_external);
+
 	g_test_add_func("/install/image-selection/readonly", test_install_image_readonly);
 
 	g_test_add_func("/install/image-mapping/variants", test_install_image_variants);
