@@ -8,12 +8,12 @@
 #include "common.h"
 #include "context.h"
 #include "mount.h"
+#include "utils.h"
 
 typedef struct {
 	gchar *tmpdir;
 } UpdateHandlerFixture;
 
-#define BIT(nr) (1UL << (nr))
 typedef enum {
 	TEST_UPDATE_HANDLER_DEFAULT       = 0,
 	TEST_UPDATE_HANDLER_EXPECT_FAIL   = BIT(0),
@@ -280,6 +280,13 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 			g_test_skip("RAUC_TEST_MTD_NAND undefined");
 			return;
 		}
+	} else if (g_strcmp0(test_pair->slottype, "nor") == 0) {
+		slotpath = g_strdup(g_getenv("RAUC_TEST_MTD_NOR"));
+		if (!slotpath) {
+			g_test_message("no MTD NOR device for testing found (define RAUC_TEST_MTD_NOR)");
+			g_test_skip("RAUC_TEST_MTD_NOR undefined");
+			return;
+		}
 	} else {
 		slotpath = g_build_filename(fixture->tmpdir, "rootfs-0", NULL);
 	}
@@ -414,7 +421,7 @@ int main(int argc, char *argv[])
 
 		{"raw", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"ext4", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0},
+		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 
 		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 2},
@@ -479,6 +486,9 @@ int main(int argc, char *argv[])
 		/* nand tests */
 		{"nand", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 
+		/* nor tests */
+		{"nor", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+
 		{0}
 	};
 	setlocale(LC_ALL, "C");
@@ -838,6 +848,13 @@ int main(int argc, char *argv[])
 			update_handler_fixture_set_up,
 			test_update_handler,
 			update_handler_fixture_tear_down);
+	/* nor tests */
+	g_test_add("/update_handler/update_handler/img_to_nor",
+			UpdateHandlerFixture,
+			&testpair_matrix[55],
+			update_handler_fixture_set_up,
+			test_update_handler,
+			update_handler_fixture_tear_down);
 
 	return g_test_run();
 }
